"""Subclass of ConstructorPane, which is generated by wxFormBuilder."""

#import wx


MARGIN_LINE_NUMBERS = 0
MARGIN_FOLD = 1

import wx
import wx.stc
import wx.lib.newevent
from ctx import theContext as context
import tran
from app.ui import pane
from activity.models.ui import ui as ui
from activity.models.ui.dlg.cc import NavigatorDialog
from model import cc
from analytic.cppparse import gather_vars


#define update sash required event
UglySashEvent, EVT_UGLY_SASH = wx.lib.newevent.NewEvent()

LocalCommand, EVT_LOCAL_COMMAND_EVENT = wx.lib.newevent.NewCommandEvent()

CLOSE_PANE_ID = wx.ID_HIGHEST + 100


# Implementing ConstructorPane
class ConstructorPane(ui.ConstructorPane):
    """Pane for constructor edition"""
    def __init__(self, parent, mainframe, constructor):
        """Initialization"""
        super(ConstructorPane, self).__init__(parent)
        self._mainframe = mainframe
        self._object = constructor
        self._notebook = parent
        container = constructor.inner_member_container
        container_cls = container.inner_class
        if container_cls:
            s = [container_cls.scoped]
            scoped = lambda x: (hasattr(x, 'scoped_str') and x.scoped_str(s)
            ) or (hasattr(x, 'scoped') and x.scoped) or x.name
            self._types = dict([scoped(x), x] for x in container.types)
        else:
            self._types = dict([scoped(x), x] for x in container.types)
            scoped = lambda x: (hasattr(x, 'scoped') and x.scoped) or x.name
            self._types = dict([scoped(x), x] for x in container.types)

        if '@' in self._types:
            del self._types['@']

        self._bodyTouched = False

        self.m_editor.SetText(constructor._content)
        self.m_init.SetText(constructor._init)
        self.m_editor.Initialize(self._types)
        self.m_init.Initialize(self._types)
        self.RecalcSashPos()

        frame = context.frame
        #bind some messages to this frame
        self.Bind(wx.EVT_MENU, frame.CloseCurrentDocPane, id=CLOSE_PANE_ID)
        self.Bind(wx.EVT_MENU, self.OnDeleteCode, id=self.m_delete.GetId())
        self.Bind(wx.EVT_MENU, self.OnUndoCode, id=frame.undo.GetId())
        self.Bind(wx.EVT_MENU, self.OnRedoCode, id=frame.redo.GetId())
        self.Bind(wx.EVT_MENU, self.OnCopyCode, id=frame.copy.GetId())
        self.Bind(wx.EVT_MENU, self.OnPasteCode, id=frame.paste.GetId())
        self.Bind(wx.EVT_MENU, self.OnCutCode, id=frame.cut.GetId())
        self.Bind(wx.EVT_MENU, self.OnSaveCode, id=frame.saveProject.GetId())

        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateDeleteCode, id=self.m_delete.GetId())
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUndoCode, id=frame.undo.GetId())
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateRedoCode, id=frame.redo.GetId())
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCopyCode, id=frame.copy.GetId())
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdatePasteCode, id=frame.paste.GetId())
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCutCode, id=frame.cut.GetId())
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateSaveCode, id=frame.saveProject.GetId())

        self._selected = None
        self.m_editor.popup_handler = self
        self.m_init.popup_handler = self
        self._fullScreen = None

        wx.stc.EVT_STC_CHANGE(self.m_editor, self.m_editor.GetId(),
             self.OnEditorChange)
        wx.stc.EVT_STC_CHANGE(self.m_init, self.m_init.GetId(), self.OnInitChange)

        wx.stc.EVT_STC_ZOOM(self.m_editor, self.m_editor.GetId(),
            self.OnEditorZoomed)
        wx.stc.EVT_STC_ZOOM(self.m_init, self.m_init.GetId(),
            self.OnInitZoomed)

        EVT_UGLY_SASH(self, self.OnUglySash)

        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('F'), ui.ID_TOGGLE_FULLSCREEN),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('W'), CLOSE_PANE_ID),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('S'), frame.saveProject.GetId()),
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_DELETE, self.m_delete.GetId())
        ])
        self.SetAcceleratorTable(aTable)

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        line = self.m_init.GetFirstVisibleLine()
        pos = self.m_init.GetCurrentPos()
        self.m_init.Freeze()
        self.m_init.SetText(self._object._init)
        self.m_init.SetModified(False)
        try:
            self.m_init.SetFirstVisibleLine(line)
            self.m_init.SetCurrentPos(pos)
        except:
            pass
        self.m_init.Thaw()
        line = self.m_editor.GetFirstVisibleLine()
        pos = self.m_editor.GetCurrentPos()
        self.m_editor.Freeze()
        self.m_editor.SetText(self._object._content)
        self.m_editor.SetModified(False)
        try:
            self.m_editor.SetFirstVisibleLine(line)
            self.m_editor.SetCurrentPos(pos)
        except:
            pass
        self.m_editor.Thaw()

    def PreDelete(self):
        """Remove toolbar first in order to avoid gtk-collision at close"""
        #self.m_editor.Destroy()
        #self.m_init.Destroy()
        self.m_auiToolBar5.Destroy()

    def Popup(self, editor, text=None):
        """Called from editor to show user menus"""
        pos = editor.GetCurrentPos()
        code = editor.GetText()[:pos]
        if text:
            # in the future, we need to show a complete floating dialog here
            # we receive "this->" or "object." here ..
            _vars = gather_vars(code, self._types)
            if not _vars:
                return
            if text not in _vars:
                return
            var = _vars[text]
            _type = var['type']
            if type(_type) is cc.Class:
                pos = editor.GetCurrentPos()
                p = editor.PointFromPosition(pos)
                p = editor.ClientToScreen(p)
                dlg = NavigatorDialog(editor, _type, p)
                if dlg.ShowModal() == wx.ID_OK:
                    editor.AddText(dlg.value)
            return
        else:
            menu = wx.Menu()
            menu.Append(self._mainframe.editProperties.GetId(),
                "test menu", "Edit relation properties")
            pos = editor.GetCurrentPos()
            p = editor.PointFromPosition(pos)
            editor.PopupMenu(menu, p)
            menu.Destroy()


    def OnInitPane(self):
        """Initialization"""
        self.RecalcSashPos()

    def RecalcSashPos(self):
        """Take amodelount of init section and recalc sash"""
        h = 10
        if self.m_init.GetLineCount():
            h += (self.m_init.GetLineCount() + 1) * (self.m_init.TextHeight(0) +
                self.m_init.GetExtraAscent() +
                self.m_init.GetExtraDescent())
        #self.m_splitter2.SetMinimumPaneSize(h)
        self.m_splitter2.SetSashPosition(h, True)
        self.Layout()
        self.m_splitter2.UpdateSize()

    def Disabled(self, event):
        """Joker event"""
        event.Enable(False)

    def UpdateModifiedTitle(self):
        """Update the tile after modifications"""
        if not self._fullScreen:
            i = self._notebook.GetPageIndex(self)
            s = self._notebook.GetPageText(i)
            m = '[modified] '
            if self.m_editor.IsModified() or self.m_init.IsModified():
                if s.find(m) == wx.NOT_FOUND:
                    self._notebook.SetPageText(i, m + s)
            else:
                if s.find(m) != wx.NOT_FOUND:
                    s = s.replace(m, '')
                    self._notebook.SetPageText(i, s)

    def OnEditorChange(self, event):
        """Called when editor status change"""
        self.UpdateModifiedTitle()

    def OnInitChange(self, event):
        """Called when editor status change"""
        self.RecalcSashPos()
        self.UpdateModifiedTitle()

    def OnUglySash(self, event):
        """Recalc sash"""
        sz = event.source.GetZoom()
        tz = event.target.GetZoom()
        if sz != tz:
            event.target.SetZoom(sz)
            wx.PostEvent(self, event)
        else:
            self.RecalcSashPos()

    def OnEditorZoomed(self, event):
        """Called when the UI of the editor changes"""
        e = UglySashEvent(source=self.m_editor, target=self.m_init)
        wx.PostEvent(self, e)
        event.Skip()

    def OnInitZoomed(self, event):
        """Called when the UI of the init par changes"""
        e = UglySashEvent(source=self.m_init, target=self.m_editor)
        wx.PostEvent(self, e)
        event.Skip()

    @tran.TransactionalMethod('edit constructor')
    def Commit(self):
        """Update document"""
        text = self.m_editor.GetText()
        init = self.m_init.GetText()
        o = self._object
        if text == o._content and init == o._init:
            return False
        if text != o._content:
            o._bodyTouched = True
        if init != o._content:
            o._initTouched = True
        o.SaveState()
        o._content = text
        o._init = init
        o.project.SetModified(True)
        k = o.outer_class or o.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)
        return True

    def OnUndoCode(self, event):
        """Edit undo"""
        if self._selected:
            self._selected.Undo()

    def OnRedoCode(self, event):
        """Edit redo"""
        if self._selected:
            self._selected.Redo()

    def OnUpdateUndoCode(self, event):
        """Edit undo?"""
        if self._selected and self._selected.CanUndo():
            event.SetText("Undo edit")
            event.Enable(True)
        else:
            event.SetText("Can't undo")
            event.Enable(False)

    def OnUpdateRedoCode(self, event):
        """Edit redo?"""
        if self._selected and self._selected.CanRedo():
            event.SetText("Redo edit")
            event.Enable(True)
        else:
            event.SetText("Can't redo")
            event.Enable(False)

    def OnUpdateCopyCode(self, event):
        """Edit copy?"""
        if self._selected:
            (b, e) = self._selected.GetSelection()
            event.Enable(b != e)
        else:
            event.Enable(False)

    def OnUpdatePasteCode(self, event):
        """Edit paste?"""
        event.Enable(bool(self._selected and self._selected.CanPaste()))

    def OnUpdateCutCode(self, event):
        """Edit paste?"""
        """Edit cut?"""
        if self._object._readOnly or not self._selected:
            event.Enable(False)
        else:
            (b, e) = self._selected.GetSelection()
            event.Enable(b != e)

    def OnUpdateDeleteCode(self, event):
        """Edit delete?"""
        event.Enable(not self._object._readOnly)

    def OnUpdateSaveCode(self, event):
        """Handles update event"""
        event.Enable(self.m_editor.IsModified() or self.m_init.IsModified())

    def OnDeleteCode(self, event):
        """Handle delete for focus editor"""
        if self._selected is None:
            return
        (b, e) = self._selected.GetSelection()
        if b == e:
            self._selected.CharRight()
        self._selected.DeleteBack()

    def OnCopyCode(self, event):
        """Handle copy for focus editor"""
        if self._selected is None:
            return
        self._selected.Copy()

    def OnPasteCode(self, event):
        """Handle paste for focus editor"""
        if self._selected is None:
            return
        self._selected.Paste()

    def OnCutCode(self, event):
        """Handle paste for focus editor"""
        if self._selected is None:
            return
        self._selected.Cut()

    def OnSaveCode(self, event):
        """Handle save  event"""
        self.Commit()
        self.m_editor.ResetModified()
        self.m_init.ResetModified()
        self.UpdateModifiedTitle()

    def OnFullScreen(self, event):
        """ShowFullscreen"""
        curs = self._selected
        if self._fullScreen:
            self._fullScreen.LeaveFullScreen()
            self._fullScreen.Destroy()
            self._fullScreen = None
            self.UpdateModifiedTitle()
        else:
            self._fullScreen = pane.FullScreen(self)
        if curs:
            curs.SetFocus()

    def OnInitLeaveFocus(self, event):
        "Handle LeaveFocus event for init"
        self._selected = None
        event.Skip()

    def OnInitGetFocus(self, event):
        "Handle GetFocus event for init"
        self._selected = self.m_init
        event.Skip()

    def OnEditorLeaveFocus(self, event):
        "Handle LeaveFocus event for editor"
        self._selected = None
        event.Skip()

    def OnEditorGetFocus(self, event):
        "Handle GetFocus event for editor"
        self._selected = self.m_editor
        event.Skip()




"""Subclass of NewFolder, which is generated by wxFormBuilder."""

import wx
import wxx
import app.resources as rc
from activity.tasks.ui import ui


# Implementing NewFolder
class TaskFolderDialog(ui.NewFolder):
    """
    This dialog allows to setup the task folder name.
    Folders are organizative items for classify the
    tasks.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialization"""
        super(TaskFolderDialog, self).__init__(parent)
        self._container = container
        self.original = ""
        self.m_textCtrl2.SetFocus()
        self.m_sdbSizer2OK.SetDefault()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("folder"))
        self.SetIcon(icon)

    def Validate(self):
        """Process OnOk event"""
        self._name = self.m_textCtrl2.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("Task folder name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._note = self.m_richText3.GetValue()
        return True

    def CopyAttributes(self, folder):
        """Transfer attributes to folder"""
        folder._name = self._name
        folder.note = self._note

    def SetAttributes(self, folder):
        """Transfer attributes from existing constructor"""
        self._container = folder.parent
        self._name = folder._name
        self.original = folder._name
        self._note = folder.note
        self.m_textCtrl2.SetValue(self._name)
        self.m_richText3.SetValue(self._note)
        self.SetTitle("Edit task folder")

    def get_kwargs(self):
        """return arguments suitable for object instance"""
        return {'parent': self._container, 'name': self._name,
            'note': self._note}

    def OnOK(self, event):
        """On Ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)








# -*- coding: utf-8 -*-
"""
Created on Mon Dec 16 08:28:25 2013

@author: mel
"""


import model


class Member(model.TComponent):
    """Implements member"""
    def __init__(self, **kwargs):
        "Initialization"
        super(Member, self).__init__(**kwargs)
        topClass = self.outer_class
        topClass._lastSrcTime = None
        topClass._lastHdrTime = None

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        return super(Member, self).get_kwargs()




from ClassDiagram import ClassDiagramDialog
from ContextItems import ContextItems
from GenerateSources import GenerateSourcesDialog
from Note import NoteDialog
from Project import ProjectDialog
from cc import *
from py import *



"""Subclass of FilesView, which is generated by wxFormBuilder."""

import re
import os
import wx
import wxx
import app
import tran
import model
import ostools
import deco
import app.resources as rc
from app.ui.view import BaseView
from activity.files.ui import ui, pane, dlg
from activity.files.actions import debugPythonSession
from activity.files.actions import build
from ctx import localcontext as context


class FilesView(BaseView, ui.FilesView):
    """Implements pane for file view"""
    clipboard_handler = False
    run_handler = True
    debug_handler = True
    build_handler = True

    perspective = ''
    _refreshFilesId = wx.Window.NewControlId()
    _addProjectFileId = wx.Window.NewControlId()
    _addProjectDirId = wx.Window.NewControlId()

    def __init__(self, parent, root=None):
        """Initialize pane"""
        super(FilesView, self).__init__(parent)
        self.frame = parent
        self.imglist = ostools.MimeHandler.imageList
        self.m_tree.SetImageList(self.imglist)
        self._create_menus()
        self._create_toolbars()
        self._bind_events()
        self._set_accelerators()
        self.m_tree.AddRoot('Projects')
        self.selected = None
        # when we create a models view, we need to update elements
        if root is None:
            # reload full stack
            theApp = context.app
            for wrk in theApp.workspaces:
                self.insert(wrk)
        # enhancement for popup subviews
        else:
            self.insert(root)

    def insert(self, element):
        """Nested insert elements in tree"""
        self.DoRenderAddElement(element)
        for cls in element._child:
            for k in element[cls]:
                self.insert(k)

    def _create_menus(self):
        """Create the view menus"""
        super(FilesView, self)._create_menus()
        self._menu = wx.Menu()
        # add directory
        self._addDir = wx.MenuItem(self._menu, self._addProjectDirId, u"Add new directory ...\tCtrl+N",
            u"add new file", wx.ITEM_NORMAL)
        self._addDir.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FOLDER, wx.ART_MENU))
        self._menu.AppendItem(self._addDir)
        self._menu.AppendSeparator()
        # add file
        self._addFile = wx.MenuItem(self._menu, self._addProjectFileId, u"Add new file ...\tCtrl+N",
            u"add new file", wx.ITEM_NORMAL)
        self._addFile.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_NEW, wx.ART_MENU))
        self._menu.AppendItem(self._addFile)
        self._menu.AppendSeparator()
        # refresh files
        self._refreshFiles = wx.MenuItem(self._menu, self._refreshFilesId, u"Refresh files",
            u"refresh files", wx.ITEM_NORMAL)
        self._refreshFiles.SetBitmap(wx.Bitmap(u"app/res/refresh.xpm", wx.BITMAP_TYPE_ANY))
        self._menu.AppendItem(self._refreshFiles)
        self.RegisterMenu('Files', self._menu)

    def _create_toolbars(self):
        """Create the toolbar menus"""
        super(FilesView, self)._create_toolbars()

    def _bind_events(self):
        """Bind events to this view"""
        self.m_tree.Bind(wx.EVT_RIGHT_DOWN, self.OnTreeMenu)
        self.m_tree.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnTreeOpenFile)
        self.m_tree.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnTreeSelChanged)
        # add files
        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddFiles, id=self._addProjectFileId)
        self.BindSpecial(wx.EVT_MENU, self.OnAddFiles, id=self._addProjectFileId)
        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddDir, id=self._addProjectDirId)
        self.BindSpecial(wx.EVT_MENU, self.OnAddDir, id=self._addProjectDirId)
        # refresh files
        self.BindSpecial(wx.EVT_MENU, self.OnRefreshAllFiles, id=self._refreshFilesId)

        # debug
        self.BindSpecial(wxx.EVT_DEBUGGER, self.OnDebugEvent)

        super(FilesView, self)._bind_events()

    def _set_accelerators(self):
        """Set the accelerator table"""
        # ctrl_alt = wx.ACCEL_CTRL + wx.ACCEL_ALT
        # ctrl_shf = wx.ACCEL_CTRL + wx.ACCEL_SHIFT
        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('N'), self._addProjectFileId),
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_LEFT, BaseView._leftKeyId)
        ])
        self.SetAcceleratorTable(aTable)

    @deco.classproperty
    def name(cls):
        """returns the name of this view"""
        return 'Files'

    @classmethod
    def bitmap_index(cls):
        """return the bitmap index"""
        return rc.GetBitmapIndex('files')

    @classmethod
    def bitmap(cls):
        """return the bitmap"""
        return rc.GetBitmap('files')

    def DoRenderAddElement(self, obj):
        """Inserts element in tree"""
        # model tree
        treeOrder = [
            model.Workspace, model.Project,
            model.file.Dir, model.file.File
            ]
        if not obj._visibleInTree or type(obj) not in treeOrder:
            return
        ti = treeOrder.index(type(obj))
        #check parent
        if self.m_tree.HoldsObject(obj.parent):
            p = obj.parent
        else:
            p = self.m_tree.GetRootItem()
        if obj.parent is not None:
            # find some major friend item with the same class
            tribal = obj.parent[type(obj)]
            index = tribal.index(obj)
            pre = None
            while index > 0 and pre is None:
                index = index - 1
                candidate = tribal[index]
                if not self.m_tree.HoldsObject(candidate):
                    continue
                pre = candidate
            if pre is not None:
                self.m_tree.InsertItem(p, pre, obj.label,
                    obj.bitmap_index, obj.bitmap_index, obj)
                if hasattr(obj, 'bitmap_open_index'):
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
                if type(obj) is model.cc.Constructor:
                    self.m_tree.SetItemBold(obj, bold=obj.IsPreferred())
                elif type(obj) in [model.py.Module, model.py.Package]:
                    self.m_tree.SetItemBold(obj, bold=obj._entry)
                return
        itemCount = 0
        citem, cookie = self.m_tree.GetFirstChild(p)
        if type(citem) is wx.TreeItemId:
            citem = self.m_tree.__fer__(citem)
        if type(citem) in treeOrder:
            if ti <= treeOrder.index(type(citem)):
                self.m_tree.PrependItem(p, obj.label,
                     obj.bitmap_index, obj.bitmap_index, obj)
                if hasattr(obj, 'bitmap_open_index'):
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
                if type(obj) is model.cc.Constructor:
                    self.m_tree.SetItemBold(obj, bold=obj.IsPreferred())
                elif type(obj) in [model.py.Module, model.py.Package]:
                    self.m_tree.SetItemBold(obj, bold=obj._entry)
                return
        while type(citem) is not wx.TreeItemId or citem.IsOk():
            itemCount = itemCount + 1
            citem, cookie = self.m_tree.GetNextChild(p, cookie)
            if type(citem) not in treeOrder:
                continue
            if ti <= treeOrder.index(type(citem)):
                self.m_tree.InsertItemBefore(p, itemCount,
                     obj.label,
                     obj.bitmap_index, obj.bitmap_index, data=obj)
                if hasattr(obj, 'bitmap_open_index'):
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index,
                        wx.TreeItemIcon_SelectedExpanded)
                if type(obj) is model.cc.Constructor:
                    self.m_tree.SetItemBold(obj, bold=obj.IsPreferred())
                elif type(obj) in [model.py.Module, model.py.Package]:
                    self.m_tree.SetItemBold(obj, bold=obj._entry)
                return
        #Ok, do apppend
        self.m_tree.AppendItem(p, obj.label, obj.bitmap_index,
            obj.bitmap_index, obj)
        if hasattr(obj, 'bitmap_open_index'):
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
            self.m_tree.SetItemImage(obj, obj.bitmap_open_index,
                wx.TreeItemIcon_SelectedExpanded)
        if type(obj) is model.cc.Constructor:
            self.m_tree.SetItemBold(obj, bold=obj.IsPreferred())
        elif type(obj) in [model.py.Module, model.py.Package]:
            self.m_tree.SetItemBold(obj, bold=obj._entry)

    def UpdateElement(self, obj):
        """Update the tree label for a object"""
        if not self.m_tree.HoldsObject(obj):
            return
        self.m_tree.SetItemText(obj, obj.label)
        self.m_tree.SetItemImage(obj,
            obj.bitmap_index, wx.TreeItemIcon_Normal)
        if hasattr(obj, 'bitmap_open_index'):
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
        self.m_tree.SetItemImage(obj,
            obj.bitmap_index, wx.TreeItemIcon_Selected)
        if type(obj) is model.cc.Constructor:
            self.m_tree.SetItemBold(obj, bold=obj.IsPreferred())
        elif type(obj) in [model.py.Module, model.py.Package]:
            self.m_tree.SetItemBold(obj, bold=obj._entry)

    def OnUpdateAddFiles(self, event):
        """Update event handler"""
        event.Enable(bool(self.selected and self.selected.project))

    @tran.TransactionalMethod('add new file {}')
    @wxx.CreationDialog(dlg.NewFile, model.file.File)
    def OnAddFiles(self, event):
        """Event handler"""
        args = [model.file.Dir, model.Project]
        return (context.frame, self.selected.inner(args))

    def OnUpdateAddDir(self, event):
        """Update event handler"""
        event.Enable(bool(self.selected and self.selected.project))

    @tran.TransactionalMethod('add new dir {}')
    @wxx.CreationDialog(dlg.NewDir, model.file.Dir)
    def OnAddDir(self, event):
        """Event handler"""
        container = self.selected.inner(model.file.Dir)
        if not container:
            container = self.selected.project
        return (context.frame, container)

    def OnRefreshAllFiles(self, event):
        """Event handler for refreshing all files"""
        if self.selected:
            t = type(self.selected)
            if t is model.Workspace:
                self.RefreshAllFiles()
            else:
                self.selected.project.RefreshProjectFiles()
        else:
            self.RefreshAllFiles()

    @tran.TransactionalMethod('refresh all files')
    def RefreshAllFiles(self):
        """Refresh all the files"""
        theApp = context.app
        for wrk in theApp.workspaces:
            self.RefreshWorkspaceFiles(wrk)
        return True

    def RefreshWorkspaceFiles(self, _workspace):
        """Refresh the files contained in a single workspace"""
        for prj in _workspace(model.Project):
            prj.RefreshProjectFiles()

    def OnTreeSelChanged(self, event):
        """Called when tree selection changes"""
        sel = self.m_tree.GetSelection()
        self.selected = (type(sel) is not wx.TreeItemId and sel) or None

    def DoRenderRemoveElement(self, obj):
        """Do remove element in tree"""
        if self.m_tree.HoldsObject(obj):
            self.m_tree.Delete(obj)

    @tran.TransactionalMethod('delete')
    def OnDelete(self, event):
        """delete element"""
        import tran.TransactionStack as stack
        obj = self.selected
        if type(obj) is model.file.Dir:
            stack.instance.SetName('delete dir {}'.format(obj.name))
        elif type(obj) is model.file.File:
            stack.instance.SetName('delete file {}'.format(obj.name))
        obj.Delete()
        return True

    def TreeLeftKey(self, event):
        """If the selected node is expanded, simply collapse it.
        If not, navigate through parent"""
        if not self.selected:
            return
        if self.m_tree.IsExpanded(self.selected):
            self.m_tree.Collapse(self.selected)
        else:
            parent = self.selected.parent
            if self.m_tree.HoldsObject(parent):
                self.m_tree.SelectItem(parent)

    def OnTreeMenu(self, event):
        """Handles context tree popup menu"""
        (item, where) = self.m_tree.HitTest(event.GetPosition())
        if item is None:
            return
        self.m_tree.SelectItem(item)
        obj = self.selected
        if obj is None:
            return
        menu = wx.Menu()
        menu.Append(app.ID_EDIT_OPEN, "open editor", "Edit file.")
        # add elements filtering disabled
        app.ui.clone_mnu(self._menu, parent=menu, enabled=True)
        menu.AppendSeparator()
        app.ui.append_menuitem_copy(menu, self.frame.editProperties)
        self.PopupMenu(menu, event.GetPosition())
        menu.Destroy()

    def OnTreeOpenFile(self, event):
        """Handle open file"""
        obj = self.selected
        if obj is None or type(obj) is not model.file.File:
            return
        if hasattr(obj, '_pane') and obj._pane is not None:
            try:
                self.frame.docBook.SetSelection(
                    self.frame.docBook.GetPageIndex(obj._pane))
                return
            except:
                print "spureus _pane reference detected"
                del obj._pane
        frame = context.frame
        # get the appropiated editor
        stat = os.stat(obj.abs_file)
        if stat.st_size > 0:
            access = os.access(obj.abs_file, os.R_OK)
        else:
            access = os.access(obj.abs_file, os.W_OK)
        if not access:
            wx.MessageBox("The file is not readable", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        if re.match(r"(.)*\.cpp$", obj._file) or re.match(r"(.)*\.h$", obj._file):
            frame.OpenDoc(pane.FilePane, obj)
            return
        if re.match(r"(.)*\.py$", obj._file):
            frame.OpenDoc(pane.FilePythonPane, obj)
            return
        if re.match(r"(.)*\.txt$", obj._file):
            frame.OpenDoc(pane.TextPane, obj)
            return
        if re.match(r"(.)*\.pdf$", obj._file):
            import app.ui.pane as apane
            frame.OpenDoc(apane.PdfViewPane, obj)
            return
        # fallback to text file if not binary
        try:
            if not obj.is_binary:
                frame.OpenDoc(pane.TextPane, obj)
        except:
            pass

    def OnUpdateEditProperties(self, event):
        """"""
        event.Enable(False)

    def OnEditProperties(self, event):
        """"""
        pass

    def OnUpdateDelete(self, event):
        """"""
        if hasattr(self, '_dbg_session'):
            event.Enable(False)
        elif self.selected:
            if self.selected._readOnly:
                event.Enable(False)
                return
            t = type(self.selected)
            if t is model.file.File:
                event.Enable(True)
                return
            if t is not model.file.Dir:
                event.Enable(False)
                return
            if any(x for x in self.selected(model.file.File,
                model.file.Dir) if x._readOnly):
                event.Enable(False)
                return
            event.Enable(True)
        else:
            event.Enable(False)

    def OnUpdateRunFile(self, event):
        """default event handler"""
        if not hasattr(self.frame, '_runProcess'):
            obj = self.selected
            event.Enable(bool(obj and type(obj) is model.file.File
                and re.match(r"(.)*\.py", obj.abs_file)))
        else:
            event.Enable(False)

    def OnUpdateToggleBreakpoint(self, event):
        """Update toggle"""
        book = self.frame.docBook
        if book.GetPageCount():
            page = book.GetCurrentPage()
            if type(page) in [pane.FilePane, pane.FilePythonPane]:
                event.Enable(True)
                event.Check(page.breakpoint)
                return
        event.Enable(False)

    def OnToggleBreakpoint(self, event):
        """Toggle breakpoint"""
        book = self.frame.docBook
        if not book.GetPageCount():
            return
        page = book.GetCurrentPage()
        if type(page) not in [pane.FilePane, pane.FilePythonPane]:
            return
        line = page.line
        page.ToggleBreakpoint(event)
        if hasattr(self, '_dbg_session'):
            pfile = page._object
            if page.breakpoint:
                cmd = 'break {file}:{bp}'.format(file=pfile._file, bp=line + 1)
                self._dbg_session.send_command(cmd)
            else:
                # This is ugly but my time is small.
                # we must do it better ... pending
                self._dbg_session.send_command('delete')
                self.UpdateBreakpoins()
            self._dbg_session.send_command('info break')

    def OnUpdateDebugFile(self, event):
        """default event handler"""
        if hasattr(self, '_dbg_session'):
            event.Enable(False)
        else:
            obj = self.selected
            event.Enable(bool(obj and type(obj) is model.file.File
                and re.match(r"(.)*\.py", obj._file)))

    def OnDebugEvent(self, event):
        """handle debugger events dispatched from client"""
        if event.wich == wxx.USER_COMMAND_RESPONSE:
            if hasattr(self, '_dbg_session'):
                response = event.message
                self._dbg_session.UpdateDebugCommandUI(response)
        elif event.wich == wxx.FILE_LINE_INFO:
            path, line = event.message
            self.OpenPythonFileLine(path, line)
        elif event.wich == wxx.UNKNOWN_DEBUG_INFO:
            pass
        elif event.wich == wxx.DEBUG_ENDED:
            if hasattr(self, '_dbg_session'):
                self._dbg_session.deactivateUI()
                del self._dbg_session
            # restore editors
            for i in range(0, self.frame.docBook.GetPageCount()):
                pane = self.frame.docBook.GetPage(i)
                if hasattr(pane, 'RestoreReadOnly'):
                    pane.RestoreReadOnly()
        elif event.wich == wxx.UPDATE_THREADS_INFO:
            if hasattr(self, '_dbg_session'):
                self._dbg_session.UpdateThreadsUI()
        elif event.wich == wxx.UPDATE_LOCALS_INFO:
            if hasattr(self, '_dbg_session'):
                self._dbg_session.UpdateLocalsUI()
        elif event.wich == wxx.UPDATE_BREAKPOINTS_INFO:
            if hasattr(self, '_dbg_session'):
                self._dbg_session.UpdateBreakpointsUI()

    def OpenPythonFileLine(self, path, line):
        """Open a file by path"""
        # search the file inside workspaces
        p = os.path.abspath(path)
        for wrk in context.app.workspaces:
            items = wrk(model.file.File, filter=lambda x: x.abs_file == p)
            if not len(items):
                continue
            # Ordenamos la apertura del fichero
            obj = items[0]
            if hasattr(obj, '_pane') and obj._pane is not None:
                self.frame.docBook.SetSelection(
                    self.frame.docBook.GetPageIndex(obj._pane))
            else:
                self.frame.OpenDoc(pane.FilePythonPane, obj)
            obj._pane.SetReadOnly(True)
            obj._pane.GotoLine(line - 1, True)
            return True
        return False

    def OnBuildProject(self, event):
        """Update build project"""
        # before build, save first!
        book = self.frame.docBook
        n = book.GetPageCount()
        for pane in [book.GetPage(i) for i in range(0, n)]:
            try:
                if pane.object.project is self.selected:
                    pane.Commit()
            except:
                pass
        build(self.selected.project)

    def OnUpdateDebugProject(self, event):
        """Update the run project button"""
        if hasattr(self, '_dbg_session'):
            event.Enable(False)
        else:
            v = self.selected
            if v and hasattr(v, 'project'):
                v = v.project
                if v and v._language == 'python' and v.main_file:
                    v = v.main_file
                    event.Enable(os.path.exists(v.key_file))
                    return
        event.Enable(False)

    def OnDebugProject(self, event):
        """default event handler"""
        # before running, save first!
        book = self.frame.docBook
        n = book.GetPageCount()
        for pane in [book.GetPage(i) for i in range(0, n)]:
            try:
                if pane.object.project is self.selected:
                    pane.Commit()
            except:
                pass
        self._dbg_session = debugPythonSession(self.selected.project.main_file.key_file)
        self.UpdateBreakpoins()
        self._dbg_session.send_command('info line')
        self._dbg_session.send_command('info threads verbose')
        self._dbg_session.send_command('pp locals()')
        self._dbg_session.send_command('info break')

    def OnDebugFile(self, event):
        """default event handler"""
        # before running, save first!
        if hasattr(self.selected, '_pane') and self.selected._pane:
            self.selected._pane.Commit()
        self._dbg_session = debugPythonSession(self.selected.abs_file)
        self.UpdateBreakpoins()
        self._dbg_session.send_command('info line')
        self._dbg_session.send_command('info threads verbose')
        self._dbg_session.send_command('pp locals()')
        self._dbg_session.send_command('info break')

    def OnUpdateDebugContinue(self, event):
        """default event handler"""
        if hasattr(self, '_dbg_session'):
            event.Enable(self._dbg_session._stepping)
        else:
            event.Enable(False)

    def OnDebugContinue(self, event):
        """default event handler"""
        self._dbg_session.send_command('continue')
        # post standard commands for finish or bp
        self._dbg_session.send_command('info line')
        self._dbg_session.send_command('info threads verbose')
        self._dbg_session.send_command('pp locals()')

    def OnUpdateDebugStepOver(self, event):
        """default event handler"""
        if hasattr(self, '_dbg_session'):
            event.Enable(self._dbg_session._stepping)
        else:
            event.Enable(False)

    def OnDebugStepOver(self, event):
        """default event handler"""
        self._dbg_session.send_command('next')
        self._dbg_session.send_command('info line')
        self._dbg_session.send_command('info threads verbose')
        self._dbg_session.send_command('pp locals()')

    def OnUpdateDebugStepInto(self, event):
        """Update debug stop"""
        if hasattr(self, '_dbg_session'):
            event.Enable(self._dbg_session._stepping)
        else:
            event.Enable(False)

    def OnDebugStepInto(self, event):
        """default event handler"""
        self._dbg_session.send_command('step')
        self._dbg_session.send_command('info line')
        self._dbg_session.send_command('info threads verbose')
        self._dbg_session.send_command('pp locals()')

    def OnUpdateDebugStepOut(self, event):
        """Update debug stop"""
        if hasattr(self, '_dbg_session'):
            event.Enable(self._dbg_session._stepping)
        else:
            event.Enable(False)

    def OnDebugStepOut(self, event):
        """default event handler"""
        self._dbg_session.send_command('frame 0')
        self._dbg_session.send_command('finish')
        self._dbg_session.send_command('info line')
        self._dbg_session.send_command('info threads verbose')
        self._dbg_session.send_command('pp locals()')

    def OnDebugStop(self, event):
        """default event handler"""
        if self._dbg_session._stepping:
            self._dbg_session.send_command('quit')
        else:
            self._dbg_session.force_stop()

    def UpdateBreakpoins(self):
        """Update the brekpoints in the debugger"""
        if not hasattr(self, '_dbg_session'):
            return
        if not self.selected:
            return
        project = self.selected.project
        pfiles = project(model.file.File)
        for pfile in pfiles:
            for bp in project.breakpoints(pfile):
                cmd = 'break {file}:{bp}'.format(file=pfile._file, bp=bp + 1)
                self._dbg_session.send_command(cmd)

    def OnUpdateDebugStop(self, event):
        """Update debug stop"""
        event.Enable(hasattr(self, '_dbg_session'))

    def OnRunFile(self, event):
        """default event handler"""
        # before running, save first!
        if hasattr(self.selected, '_pane') and self.selected._pane:
            self.selected._pane.Commit()
        self.frame.RunPythonScript(self.selected.abs_file)

    def OnUpdateStopRun(self, event):
        """default event handler"""
        event.Enable(hasattr(self.frame, '_runProcess'))

    def OnStopRun(self, event):
        """Stop running"""
        self.frame.StopRunning()

    def OnUpdateRunProject(self, event):
        """Update the run project button"""
        if not hasattr(self.frame, '_runProcess'):
            v = self.selected
            if v and hasattr(v, 'project'):
                v = v.project
                if v and v._language == 'python' and v.main_file:
                    v = v.main_file
                    event.Enable(os.path.exists(v.key_file))
                    return
        event.Enable(False)

    def OnRunProject(self, event):
        """default event handler"""
        # before running, save first!
        path = self.selected.project.main_file.key_file
        book = self.frame.docBook
        n = book.GetPageCount()
        for pane in [book.GetPage(i) for i in range(0, n)]:
            try:
                if pane.object.project is self.selected:
                    pane.Commit()
            except:
                pass
        self.frame.RunPythonScript(path)




"""Subclass of NewNamespace, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import model
import app.resources as rc


# Implementing NewNamespace
class NamespaceDialog(ui.NewNamespace):
    """
    This dialog allows to setup a namespace container.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialization"""
        super(NamespaceDialog, self).__init__(parent)
        self.container = container
        self.original = ""
        self.m_textCtrl2.SetFocus()
        self.m_sdbSizer2OK.SetDefault()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("namespace"))
        self.SetIcon(icon)

    def Validate(self):
        """Process OnOk event"""
        self._name = self.m_textCtrl2.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Namespace name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("namespace name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if self.original != self._name:
            if self._name in [x._name for x in self.container.top_namespaces]:
                wx.MessageBox("Namespace already exist", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        self._note = self.m_richText3.GetValue()
        return True

    def CopyAttributes(self, namespace):
        """Transfer attributes to namespace"""
        namespace._name = self._name
        namespace.note = self._note

    def SetAttributes(self, namespace):
        """Transfer attributes from existing constructor"""
        self._container = namespace.parent
        self._name = namespace._name
        self.original = namespace._name
        self._note = namespace.note
        self.m_textCtrl2.SetValue(self._name)
        self.m_richText3.SetValue(self._note)
        self.SetTitle("Edit namespace")

    def get_kwargs(self):
        """Return kwargs suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self.container
        kwargs['name'] = self._name
        kwargs['note'] = self._note
        return kwargs

    def OnOK(self, event):
        """On Ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-

"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""

import model
import os
import copy
from model.writer import writer
import app.resources as rc
import model.decorator as ctx
import tran
from tran import TransactionStack
from app.utils import cached_type


class Class(model.TComponent):
    """Implements c++ class representation"""
    class_container = True
    context_container = True
    folder_container = True
    diagram_container = True
    inheritance_container = True
    member_container = True
    relation_container = True
    enum_container = True
    type_container = True

    #default empty user sections
    user_code_h1 = ''
    user_code_h2 = ''
    user_code_h3 = ''
    user_code_s1 = ''
    user_code_s2 = ''
    user_code_s3 = ''

    # visual methods
    @tran.TransactionalMethod('move class {}')
    def drop(self, to):
        """Drops class inside project or another folder """
        target = to.inner_namespace_container
        if not target or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization method"""
        self._is_external = kwargs.get("is_external", False)
        self._is_struct = kwargs.get("is_struct", False)
        self._access = kwargs.get('access', "public")
        self._template = kwargs.get('template', None)
        self._template_types = kwargs.get('template_types', [])
        self._deriv = kwargs.get('derivatives', [])
        self._memberPrefix = kwargs.get('prefix', '_')
        self._lastHdrTime = None
        self._lastSrcTime = None
        self._header_obj = None  # vincle with file
        self._source_obj = None  # vincle with file
        super(Class, self).__init__(**kwargs)
        self.project.ExportCppCodeFiles(force=True)

    def set_source(self, source_obj):
        """Sets a vincle with a file"""
        self._source_obj = source_obj

    def set_header(self, header_obj):
        """Sets a vincle with a file"""
        self._header_obj = header_obj

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['is_struct'] = self._is_struct
        kwargs['access'] = self._access
        kwargs['template'] = self._template
        kwargs['template_types'] = self._template_types
        kwargs['derivatives'] = self._deriv
        kwargs['prefix'] = self._memberPrefix
        kwargs['is_external'] = self.is_external
        kwargs.update(super(Class, self).get_kwargs())
        return kwargs

    def lower(self):
        """Criteria for sorting when generating code"""
        return self._inheritance_level

    #backward compatibility code - to simplfy in newer versions
    @property
    def is_external(self):
        """return info about if the class is external"""
        if not hasattr(self, '_is_external'):
            self._is_external = False
        return self._is_external

    @property
    def virtual_bases(self):
        """Get the list of all virtual base classes"""
        # some comments: The inheritance icons will NEVER be hidden
        # by nested folders. It will be ALLWAYS be a direct child of class
        r = []
        for u in self[model.cc.Inheritance]:
            r.extend([x for x in u.virtual_bases if x not in r])
        return r

    def GetClassArguments(self):
        """Get the list of arguments required by the class"""
        # The arguments required by the class are determined following
        # the rules (in this order):
        # 1º Required arguments from virtual base classes
        # 2º Required arguments from normal base classes (not virtual)
        # 3º non-static Members without initialization
        # ** NOT: 4º Required arguments from relation parents...
        #
        # For base classes (both virtual or not), the required arguments
        # are either determined by preferred constructors, either none.
        #
        # Ok, let's go...
        # 1º & 2º.....
        vinh = self.virtual_bases
        ninh = [x._ancestor for x in self[model.cc.Inheritance] if x._ancestor not in vinh]
        # create a contributor's list
        inh = vinh
        inh.extend(ninh)
        arglist = []
        # append arguments from preferred ctors
        for x in inh:
            ctor = x.GetPreferredConstructor()
            if ctor is not None:
                for arg in ctor[model.cc.Argument]:
                    arglist.append(arg)
        # 3º append (without class prefix) non-static uninitialized members
        arglist.extend([x for x in
            self(model.cc.MemberData)
            if x.inner_class is self and x._static is False
            and len(x._default) == 0])
        # 4º append relations is not required: It must be constructed
        # from members
        # All is complete: Please note that the return list contains
        # arguments and data members, with possible duplcated types and
        # names, that require to be processed when generating the
        # initialization of constructors
        return arglist

    def UpdateClassRelations(self):
        """This method gets invoked when something changes in class relations specification
        and ensure mainteinance of internal methods and ctors"""
        self.UpdateInitMethod()
        self.UpdateExitMethod()
        self.AutoArgs()
        self.AutoInit()
        self.UpdateInitCalls()
        self.UpdateExitCalls()

    def AutoInit(self):
        """Update all init ctors"""
        for x in self(model.cc.Constructor,
            filter=lambda x: x.inner_class == self, cut=True):
            x.AutoInit()

    def UpdateInitCalls(self):
        """Update the calls to init method"""
        for x in self(model.cc.Constructor,
            filter=lambda x: x.inner_class == self, cut=True):
            x.UpdateInitCall()

    def UpdateExitCalls(self):
        """Update the calls to exit method"""
        for x in self(model.cc.Destructor, filter=lambda x: x.inner_class == self, cut=True):
            x.UpdateExitCall()

    def AutoArgs(self):
        """Update all ctor arguments"""
        for x in self(model.cc.Constructor,
            filter=lambda x: x.inner_class == self, cut=True):
            x.AutoArgs()

    def errorCheckDerivatives(self):
        """We have detected that deleted derivatives remains in
        some class object, so this method checks and warns"""
        errors = False
        n = []
        print "start check\n"
        if not hasattr(self, '_memberPrefix'):
            self._memberPrefix = '_'
        if not hasattr(self, '_is_struct'):
            self._is_struct = False
        if self._is_struct:
            cos = 'struct'
        else:
            cos = 'class'
        for cls in self._deriv:
            test = [u for u in cls[model.cc.Inheritance] if u._ancestor == self]
            if len(test) == 0:
                print "Dead inheritante from {} to {} {}\n".format(
                    self._name, cos, cls._name)
                errors = True
            else:
                n.append(cls)
        self._deriv = n
        return errors

    def OnUndoRedoRemoving(self):
        """Handle OnUndoRedoRemoving, prevent generating files"""
        stack = tran.TransactionStack
        method = self.ExportCppCodeFiles.inner
        if method not in stack.delayedCallsFiltered:
            stack.delayedCallsFiltered.append(method)
        super(Class, self).OnUndoRedoRemoving()

    @tran.DelayedMethod
    def ExportCppCodeFiles(self, force=False):
        """Do code generation"""
        self.updateSources(force)
        self.updateHeaders(force)

    @property
    def abs_path_source(self):
        """Return the absolute path of source"""
        return os.path.join(self.project.sources_dir, self._name + ".cpp")

    def updateSources(self, force=False):
        """does source generation"""
        fname = self.abs_path_source
        regenerate = False
        if force or not os.path.exists(fname) or self._lastSrcTime is None:
            regenerate = True
        elif os.path.getmtime(fname) < self._lastSrcTime:
            regenerate = True
        if not regenerate:
            return
        f = open(fname, 'w')
        pf = writer.for_file(f)
        #master include file
        prj = self.project

        pf.writeln("// {user.before.include.begin}")
        pf.writeln(self.user_code_s1)
        pf.writeln("// {user.before.include.end}")
        pf.writenl()
        if prj._useMaster:
            pf.writeln('#include "{}"'.format(prj._masterInclude))
        else:
            pf.writeln('#include "{}.h"'.format(self._name))
        pf.writenl()

        # some facility
        pf.writeln("#include <assert.h>")
        pf.writenl()

        self.WriteCode(pf)
        pf.writenl()
        f.close()

    @property
    def kind(self):
        """Returns the class flavor"""
        return (self._is_struct and "struct") or "class"

    def _ensure_access(self, pf, access):
        "utility method that ensures access update when needed"
        if access != self.contextAccess:
            self.contextAccess = access
            pf.writeln("{}:".format(access))

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the class declaration"""

        #if this is a nested class, ensure access
        pc = self.parent.inner_class
        if pc:
            pc._ensure_access(pf, self._access)

        cos = '{self.kind} {self._name}'.format(self=self)
        self.contextAccess = (self._is_struct and "public") or "private"
        self._inlines = []
        #ok, create user section
        pf.writeln("// {{user.before.{}.begin}}".format(cos))
        pf.writeln(self.user_code_h1)
        pf.writeln("// {{user.before.{}.end}}".format(cos))
        pf.writenl()

        self.WriteComment(pf)

        #create inheritance strings
        #from Inheritance import Inheritance
        if len(self[model.cc.Inheritance]):
            s = ": "
            for o in self[model.cc.Inheritance]:
                if o._virtual:
                    s += "{} virtual {},".format(o._access, o._name)
                else:
                    s += "{} {},".format(o._access, o._name)
            s = s[:-1]
        else:
            s = ""
        pf.writeln("{0} {1}{2}".format(self.kind, self._name, s))
        pf.openbrace()
        pf.writenl()
        pf.writeln("// {{user.inside.{}.begin}}".format(cos))
        pf.writeln(self.user_code_h2)
        pf.writeln("// {{user.inside.{}.end}}".format(cos))
        pf.writenl()

        #write friend declarations
        for f in self.friends:
            pf.writeln('friend {}'.format(f._target.declare))

        #write types
        _type = {'public': [], 'protected': [], 'private': []}
        f = lambda x: x.inner_class == self and getattr(x, '_definition', 'True').strip()
        for t in self(model.cc.Type, filter=f, cut=True):
            _type[t._access].append(t)
        for access in _type:
            if not _type[access]:
                continue
            pf.writeln('{}:'.format(access))
            pf.writenl()
            for t in _type[access]:
                pf.writeln('//type definition for {}'.format(t._name))
                pf.writeln('{}'.format(t._definition))
                pf.writenl()

        #we will follow and respect the visual order
        self_element = lambda x: x.parent.inner_class == self
        for element in self(model.cc.Enum, Class, model.cc.MemberData,
            model.cc.Constructor, model.cc.MemberMethod, model.cc.InitMethod,
             model.cc.ExitMethod, model.cc.IsClassMethod, model.cc.Destructor,
            filter=self_element, cut=True):
            element.WriteDeclaration(pf)

        pf.closebrace(";")
        pf.writenl()
        pf.writeln("// {{user.after.{}.begin}}".format(cos))
        pf.writeln(self.user_code_h3)
        pf.writeln("// {{user.after.{}.end}}".format(cos))
        pf.writenl()
        del self.contextAccess

    @property
    def abs_path_header(self):
        """Return the absolute path of source"""
        return os.path.join(self.project.headers_dir, self._name + ".h")

    def updateHeaders(self, force=False):
        """Realiza la generacion de fuentes"""
        fname = self.abs_path_header
        regenerate = False
        if force or not os.path.exists(fname) or self._lastHdrTime is None:
            regenerate = True
        elif os.path.getmtime(fname) < self._lastHdrTime:
            regenerate = True
        if not regenerate:
            return
        f = open(fname, 'w')
        pf = writer.for_file(f)
        #create safeguard
        safeguard = self._name.upper() + "_H_INCLUDED"
        pf.writeln("#if !defined({})".format(safeguard))
        pf.writeln("#define {}".format(safeguard))
        pf.writenl()

        #we are using master include? if not, include inheritances
        p = self.project
        if not p._useMaster and self.outer_class == self:
            #iterate over inheritances
            if len(self._child[model.cc.Inheritance]):
                pf.writeln("//base includes")
                for o in self[model.cc.Inheritance]:
                    pf.writeln("#include \"{}.h\"".format(o._name))
                pf.writenl()
            #we need to forward references for relations
            if len(self.child[model.cc.RelationFrom]):
                pf.writeln("//forward references of parent classes")
                for p in self[model.cc.RelationFrom]:
                    pf.writeln(p._FROM.reference)
            if len(self.child[model.cc.RelationTo]):
                pf.writeln("//forward references of child classes")
                for p in self[model.cc.RelationFrom]:
                    pf.writeln(p._TO.reference)
        pf.writenl()
        self.WriteDeclaration(pf)
        #end safeguard
        pf.writeln("#endif //{0}".format(safeguard))
        #write inlines
        pf.writenl()
        if len(self._inlines) > 0:
            pf.writeln("#if defined(INCLUDE_INLINES)")
            pf.writeln("#if !defined(INCLUDE_INLINES_{name})".format(name=self._name.upper()))
            pf.writeln("#define INCLUDE_INLINES_{name}".format(name=self._name.upper()))
            pf.writenl()
            for o in self._inlines:
                o.WriteCode(pf)
                pf.writenl()
            pf.writeln("#endif //(INCLUDE_INLINES_{name})".format(name=self._name.upper()))
            pf.writeln("#endif //INCLUDE_INLINES")
            pf.writenl()
        del self._inlines
        self._lastHdrTime = os.path.getmtime(fname)
        pf.writenl()
        f.close()

    @ctx.ContextImplementation()
    def WriteCode(self, pf):
        """Write code for class"""
        tag = '{self.kind}.{self._name}'.format(self=self)
        pf.writeln("// {{user.before.{tag}.begin}}".format(tag=tag))
        pf.writeln(self.user_code_s2)
        pf.writeln("// {{user.before.{tag}.end}}".format(tag=tag))
        pf.writenl()

        # we have an interesting problem here:
        # From the point of view of easy coding and track context
        #we will follow and respect the visual order
        self_member = lambda x: x.parent.inner_class == self and (not hasattr(x, '_inline') or not x._inline)
        if self.outer_class == self:
            elements = self(model.cc.MemberData, filter=lambda x: x._static)
        else:
            elements = []
        #do it with steps for ensuring order
        elements += self(model.cc.InitMethod, filter=self_member, cut=True)
        elements += self(model.cc.ExitMethod, filter=self_member, cut=True)
        elements += self(model.cc.Constructor, filter=self_member, cut=True)
        elements += self(model.cc.IsClassMethod, filter=self_member, cut=True)
        elements += self(model.cc.MemberMethod, filter=self_member, cut=True)
        elements += self(model.cc.Destructor, filter=self_member, cut=True)
        elements += self(Class, filter=self_member, cut=True)

        # obtain all the interesting elements and write
        for element in elements:
            element.WriteCode(pf)

        pf.writenl()
        pf.writeln("// {{user.after.{tag}.begin}}".format(tag=tag))
        pf.writeln(self.user_code_s3)
        pf.writeln("// {{user.after.{tag}.end}}".format(tag=tag))
        pf.writenl()

    def GetPreferredConstructor(self):
        """Obtiene el constructor por defecto o None"""
        ctors = self(model.cc.Constructor)
        for ctor in ctors:
            if ctor.IsPreferred():
                return ctor
        # if not preferred, we return the first if any
        if len(ctors):
            return ctors[0]
        return None

    def GetSerialConstructor(self):
        """Obtiene el constructor de serializacion o None"""
        for ctor in self(model.cc.Constructor):
            if ctor.IsSerial():
                return ctor
        return None

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        if len(self._deriv) > 0:
            return False
        project = self.project
        #Obtenemos la lista de miembros de clase
        if project is not None:
            #check if outer element is using this class
            outer = lambda x: self not in x.path
            typed = project(model.cc.MemberData, model.cc.MemberMethod,
                model.cc.Argument, filter=outer, cut=True)
            typed += project(model.cc.Function, model.cc.Data)
            for x in typed:
                if x._typei._type == self:
                    return False
        return super(Class, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        project = self.project
        for dia in self.project(model.ClassDiagram):
            # Check if inherit is in
            elem = dia.FindElement(self)
            if elem is not None:
                dia.SaveState()
                dia.RemoveElement(elem)
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        if hasattr(self, '_header_obj') and self._header_obj:
            self._header_obj.Delete()
        if hasattr(self, '_source_obj') and self._source_obj:
            self._source_obj.Delete()
        super(Class, self).Delete()
        project.ExportCppCodeFiles(force=True)

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(Class, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(Class, self).RestoreRelations()

    def SaveState(self):
        """Utility for saving state"""
        for rf in self[model.cc.RelationFrom]:
            rf._key.SaveState()
        for rt in self[model.cc.RelationTo]:
            rt._key.SaveState()
        super(Class, self).SaveState()

    def OnUndoRedoChanged(self):
        """Update from app"""
        #when the class is updated, ctor's and dtors, must be updated
        for ctor in self(model.cc.Constructor):
            ctor._name = self._name
            ctor.OnUndoRedoChanged()
        for dtor in self(model.cc.Destructor):
                dtor._name = self._name
                dtor.OnUndoRedoChanged()
        #inheritance labels must be changed
        for deriv in self._deriv:
            for inh in deriv[model.cc.Inheritance]:
                if inh._ancestor == self:
                    inh._name = self._name
                    inh.OnUndoRedoChanged()
        #relation labels must be changed?
        for rf in self[model.cc.RelationFrom]:
            rf._key._TO._name = "To" + self._name
            rf._key._TO.OnUndoRedoChanged()
        for rt in self[model.cc.RelationTo]:
            rt._key._FROM._name = "From" + self._name
            rt._key._FROM.OnUndoRedoChanged()
        project = self.project
        #update class diagrams
        dias = project(model.ClassDiagram)
        for dia in dias:
            # Check if class is in
            elem = dia.FindElement(self)
            if elem is not None:
                elem.Layout()
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        #update class usage as type
        typed = project(model.cc.MemberData, model.cc.MemberMethod, model.cc.Argument)
        subject = [x for x in typed if x._typei._type == self]
        # iteramos aplicando los cambios.
        for element in subject:
            element.OnUndoRedoChanged()
        super(Class, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            project.ExportCppCodeFiles(force=True)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Class, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        if not hasattr(self, '_access'):
            self._access = 'public'
        return rc.GetBitmapIndex('class', self._access)


    @property
    def tree_label(self):
        """Get tree label"""
        if self._is_struct:
            cos = 'struct'
        else:
            cos = 'class'
        if self._template:
            return 'template<{clase._template}> {cos} {clase._name}'.format(cos=cos, clase=self)
        else:
            return '{cos} {clase._name}'.format(cos=cos, clase=self)

    @property
    def declare(self):
        """"""
        if self._is_struct:
            cos = 'struct'
        else:
            cos = 'class'
        if self._template:
            return 'template<{clase._template}> {cos} {clase._name};'.format(cos=cos, clase=self)
        else:
            return '{cos} {clase._name};'.format(cos=cos, clase=self)

    def IsAncestor(self, cls):
        """Checks for ancestor relationship"""
        from model.cc import Inheritance
        if not Inheritance in cls._child:
            return False
        inhmap = cls[Inheritance]
        for inh in inhmap:
            if inh._ancestor == self:
                return True
            if self.IsAncestor(inh._ancestor):
                return True
        return False

    def ExistMemberData(self, name):
        """Check recursively about the existence of nested child member"""
        from .MemberData import MemberData
        return self.hasChild(name=name, type=MemberData)

    def UpdateInitMethod(self):
        """Update the init methods"""
        method = self.init_methods
        aggregates = [x for x in self.relations_from if x._minCardinal]
        #if there are not relations we must delete init method
        if not aggregates:
            for x in method:
                x.Delete()
            return
        #ok, if there are not method we must create it and if
        #exists, unlist it
        if not method:
            method = model.cc.InitMethod(parent=self,
                type=model.cc.typeinst(type=cached_type(self.project, 'void')))
        else:
            method = method[0]
            method.SaveState()
            # remove all the arguments from the method
            k = method[model.cc.Argument]
            while k:
                k[0].Delete()
        #ok, we need to create a map for parent types
        code = ''
        for x in aggregates:
            cls = x._FROM
            #create required arguments
            name = x._fromPtr._name
            kwargs = {
                'parent': method,
                'name': name,
                'type': copy.copy(x._fromPtr._typei),
                'note': 'pointer to parent class {}'.format(cls._name)}
            model.cc.Argument(**kwargs)
            alias = x._key._TO._name
            code += '\t{name}->Add{alias}Last(this);\n'.format(
                name=name, alias=alias)
        method._content = code

    def UpdateExitMethod(self):
        """update the exit methods"""
        method = self.exit_methods
        parent_relations = self.relations_to
        child_relations = self.relations_from
        #if there are not relations we must delete init method
        if not parent_relations and not child_relations:
            for x in method:
                x.Delete()
            return
        #ok, if there are not method we must create it and if
        #exists, unlist it
        if not method:
            method = model.cc.ExitMethod(parent=self,
                type=model.cc.typeinst(type=cached_type(self.project, 'void')))
        else:
            method = method[0]
            method.SaveState()
        # computing the contents
        # for the aggregated relations, we must delete all the elements
        code = ''
        # first of all, remove the child part
        relations = []
        aggregated = []
        for x in child_relations:
            if x._minCardinal:
                aggregated.append(x)
            else:
                relations.append(x)
        for x in relations:
            name = x._fromPtr.prefixed_name
            alias = x._key._TO._name
            s = '\tif ( {name} != nullptr )\n'
            s += '\t{\n\t\t{name}->Remove{alias}(this);\n\t}\n'
            code += s.format(name=name, alias=alias)

        for x in aggregated:
            name = x._fromPtr.prefixed_name
            alias = x._key._TO._name
            s = '\t{name}->Remove{alias}(this);\n'
            code += s.format(name=name, alias=alias)

        relations = []
        aggregated = []

        for x in parent_relations:
            if x._key._FROM._minCardinal:
                aggregated.append(x)
            else:
                relations.append(x)
        # only agreggated may be releseaded first
        for x in relations:
            name = x._firstToPtr._name
            alias = x._name
            s = '\twhile ( {name} != nullptr )\n'
            s += '\t{\n\t\tRemove{alias}({name});\n\t}\n'
            code += s.format(name=name, alias=alias)
        #ok, we need to create a map for parent types
        for x in aggregated:
            alias = x._name
            code += '\tDeleteAll{alias}();\n'.format(alias=alias)
        method._content = code

    @property
    def inner_class(self):
        """Get the inner class container"""
        return self

    @property
    def outer_class(self):
        """Get the outer class container"""
        return (self.parent and self.parent.outer_class) or self

    @property
    def friends(self):
        """returns the list of all friends annotations inside this class"""
        member = lambda x: x.inner_class == self
        return self(model.cc.Friendship, filter=member, cut=True)

    @property
    def is_class_methods(self):
        """returns the list of all is_class_methods"""
        cls_member = lambda x: x.inner_class == self
        return self(model.cc.IsClassMethod, filter=cls_member, cut=True)

    @property
    def init_methods(self):
        """returns the setup relation method"""
        cls_member = lambda x: x.inner_class == self
        return self(model.cc.InitMethod, filter=cls_member, cut=True)

    @property
    def exit_methods(self):
        """returns the clean relations method"""
        cls_member = lambda x: x.inner_class == self
        return self(model.cc.ExitMethod, filter=cls_member, cut=True)

    @property
    def nested_classes(self):
        """Returns the list of nested classes (including self)"""
        if type(self.parent) not in [model.Folder, model.cc.Class]:
            return [self]
        return self.parent.nested_classes + [self]

    @property
    def nested_types(self):
        """return the visible nested types from here"""
        visible = lambda x: x.inner_class == self or getattr(x, '_access', 'public') == 'public'
        return self(model.cc.Type, Class, filter=visible, cut=True)

    @property
    def inherited_types(self):
        """return the visible types from base classes"""
        return list(set([t for x in self[model.cc.Inheritance] if x._access != 'private'
            for t in x._ancestor.nested_types]))

    @property
    def friend_types(self):
        """return the friend classes types"""
        friends = self.project(model.cc.Friendship, filter=lambda x: x._target == self)
        return list(set([t for x in friends for t in x.inner_class.nested_types]))

    @property
    def types(self):
        """This method gets the list of visible types"""
        return list(set(self.parent.types) | set(self.friend_types) |
            set(self.inherited_types) | set(self.nested_types))

    @property
    def template_types(self):
        """Returns the list of nested template types"""
        lt = self._template_types
        nt = super(Class, self).template_types
        lt.extend([x for x in nt if x not in self._template_types])
        return lt

    @property
    def scoped(self):
        """Get the scope"""
        if self._template:
            return '{self.parent.scope}{self._name}<{self._template}>'.format(self=self)
        return '{self.parent.scope}{self._name}'.format(self=self)

    def scoped_str(self, scope_list):
        """Gets the string representation with minimal relative scope"""
        s = self.scoped
        best_match = 0
        best_scope = None
        for x in scope_list:
            if s.find(x) == 0:
                if len(x) > best_match:
                    best_match = len(x)
                    best_scope = x
        if best_scope:
            s = s[best_match:]
            if s[:2] == '::':
                s = s[2:]
        return s

    @property
    def scope(self):
        """Get the scope"""
        return '{scoped}::'.format(scoped=self.scoped)

    @property
    def reference(self):
        """Gets the forward reference"""
        if self._template:
            return 'template<{self._template}> {self.kind} {self.scoped}'.format(self=self)
        return '{self.kind} {self.scoped}'.format(self=self)

    @property
    def relations_from(self):
        """Gets the relations from"""
        cls_member = lambda x: x.inner_class == self
        return self(model.cc.RelationFrom, filter=cls_member, cut=True)

    @property
    def relations_to(self):
        """Gets the relations from"""
        cls_member = lambda x: x.inner_class == self
        return self(model.cc.RelationTo, filter=cls_member, cut=True)

    @property
    def inheritance(self):
        """Get the inheritance list"""
        cls_member = lambda x: x.inner_class == self
        return self(model.cc.Inheritance, filter=cls_member, cut=True)






"""Subclass of NewPyClass, which is generated by wxFormBuilder."""
import re
import wx
import wxx
import app.resources as rc
from activity.models.ui import ui as ui


# Implementing NewClass
class PyClassDialog(ui.NewPyClass):
    """This dialog allows to specify the python class
    attributes."""
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialice dialog"""
        super(PyClassDialog, self).__init__(parent)
        self._container = container
        self.m_textCtrl2.SetFocus()
        self.project = container.project
        self._original = ""
        #if the content is inside package, you can export it
        if not container.inner_module:
            self.m_checkBox94.SetValue(False)
            self.m_checkBox94.Enable(False)
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_class"))
        self.SetIcon(icon)

    def Validate(self):
        """implement OnOK button"""
        self._name = self.m_textCtrl2.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Class name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("class name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if not self._original == self._name:
            if self._name in [x._name for x in self.project.level_classes]:
                wx.MessageBox("Class already exist", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        self._memberPrefix = self.m_textPrefix.GetValue()
        self._export = self.m_checkBox94.IsChecked()
        self._note = self.m_richText3.GetValue()
        return True

    def CopyAttributes(self, cls):
        """Copy dialog selections to object"""
        cls._name = self._name
        cls.note = self._note
        cls._memberPrefix = self._memberPrefix
        cls._export = self._export

    def SetAttributes(self, clase):
        """Copy object attributes to dialog"""
        self.SetTitle("Edit class")
        self._original = clase._name
        self._name = clase._name
        self._note = clase._note
        self._memberPrefix = clase._memberPrefix
        self._export = clase._export

        self.m_textCtrl2.SetValue(self._name)
        self.m_textPrefix.SetValue(self._memberPrefix)
        self.m_richText3.SetValue(self._note)
        self.m_checkBox94.SetValue(self._export)

    def get_kwargs(self):
        """Returns suitable arguments for creating object"""
        kwargs = {}
        kwargs['parent'] = self._container
        kwargs['name'] = self._name
        kwargs['prefix'] = self._memberPrefix
        kwargs['note'] = self._note
        kwargs['export'] = self._export
        return kwargs

    # Handlers for NewPyClass events.
    def OnOK(self, event):
            """implement OnOK button"""
            if self.Validate():
                self.EndModal(wx.ID_OK)




"""Subclass of NewPyModule, which is generated by wxFormBuilder."""

"""Subclass of NewModule, which is generated by wxFormBuilder."""

import wx
import wxx
import app.resources as rc
from activity.models.ui import ui as ui


# Implementing NewPyModule
class PyModuleDialog(ui.NewPyModule):
    """
    This is the module setup dialog.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(PyModuleDialog, self).__init__(parent)
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_module"))
        self.SetIcon(icon)
        self._container = container

    def get_kwargs(self):
        """return arguments suitable for object instance"""
        return {'parent': self._container, 'name': self._name, 'note': self._note}

    def SetAttributes(self, obj):
        """Transfer attributes to dialog"""
        self.SetTitle("Edit module")
        self._origin = obj
        self.m_moduleName.SetValue(obj.name)
        self.m_richText1.SetValue(obj.note)

    def CopyAttributes(self, module):
        """Transfer attributes to module"""
        module._name = self._name
        module.note = self._note

    def Validate(self):
        """Process OnOk event"""
        self._name = self.m_moduleName.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("Module name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._note = self.m_richText1.GetValue()
        return True

    def OnOK(self, event):
        """On Ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



"""Subclass of NewModule, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.models.ui import ui as ui
import model
import app.resources as rc


# Implementing NewModule
class ModuleDialog(ui.NewModule):
    """
    This is the module setup dialog.
    Individual methods and variables not belonging to any class,
    must be linked to some declaration and/or implementation files.
    Some atributes,like static for variables, need a filereference
    to make sense.
    Modules apply to represent this reference. Modules may be used
    also for overriding the container files for classes, namespaces
    and structures, by simply attach they as childs of the module.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(ModuleDialog, self).__init__(parent)
        self._container = container
        # inicializamos el modulo con la lista
        # de headers y fuentes existentes
        self.m_sdbSizer9OK.SetDefault()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("module"))
        self._origin = None
        self.SetIcon(icon)

    def SetAttributes(self, obj):
        """Transfer attributes to dialog"""
        self.SetTitle("Edit module")
        self._origin = obj
        self.m_choiceHeader.SetValue(obj._header)
        self.m_choiceSource.SetValue(obj._source)
        self.m_moduleName.SetValue(obj._name)

    def CopyAttributes(self, module):
        """Transfer attributes to module"""
        module._name = self._name
        module._header = self._header
        module._source = self._source

    def Validate(self):
        """Process OnOk event"""
        self._name = self.m_moduleName.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("Module name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._header = self.m_choiceHeader.GetValue().strip()
        if len(self._header) == 0:
            wx.MessageBox("Module header must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        project = self._container.project
        #check header file coincidence
        classes = project(model.cc.Class, filter=lambda x: x.outer_class == x, cut=True)
        for cls in classes:
            if '{}.h'.format(cls._name) == self._header:
                wx.MessageBox("The header file is already used by a project class.", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        modules = project(model.cc.Module, filter=lambda x: x != self._origin, cut=True)
        for mod in modules:
            if mod._header == self._header:
                wx.MessageBox("The header file is already used by another project module.", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        #check header file name
        #fname = os.path.realpath(os.path.join(project.headers_dir, self._header))
        #if not os.path.exists(fname) and not os.access(fname, os.W_OK):
        #    wx.MessageBox("Module header file cannot be created. Please check name.", "Error",
        #        wx.OK | wx.CENTER | wx.ICON_ERROR, self)
        #    return False
        self._source = self.m_choiceSource.GetValue().strip()
        if len(self._source) == 0:
            wx.MessageBox("Module source must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        #check source file coincidence
        for cls in classes:
            if '{}.cpp'.format(cls._name) == self._source:
                wx.MessageBox("The source file is already used by a project class.", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        for mod in modules:
            if mod._source == self._source:
                wx.MessageBox("The source file is already used by another project module.", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        #check source file name
        #fname = os.path.realpath(os.path.join(project.sources_dir, self._source))
        #if not os.path.exists(fname) and not os.access(fname, os.W_OK):
        #    wx.MessageBox("Module source file cannot be created. Please check name.", "Error",
        #        wx.OK | wx.CENTER | wx.ICON_ERROR, self)
        #    return False
        return True

    def get_kwargs(self):
        """returns arguments for instance"""
        return {'parent': self._container, 'name': self._name,
            'source': self._source, 'header': self._header}

    def OnOK(self, event):
        """On Ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Jun  6 2014)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

import gettext
_ = gettext.gettext

###########################################################################
## Class LogExplorerPane
###########################################################################

class LogExplorerPane ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 511,326 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer245 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer245.AddGrowableCol( 0 )
		fgSizer245.AddGrowableRow( 0 )
		fgSizer245.SetFlexibleDirection( wx.BOTH )
		fgSizer245.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_report = wx.ListCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LC_REPORT|wx.LC_SINGLE_SEL )
		fgSizer245.Add( self.m_report, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer245 )
		self.Layout()
	
	def __del__( self ):
		pass
	




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""
import re
import app.resources as rc
from ctx import theContext as context
import model.decorator as ctx
import model
import tran
from tran import TransactionStack


class Destructor(model.Member):
    """Implements ctor method"""
    context_container = True

    # visual methods
    @tran.TransactionalMethod('move destructor ~{}')
    def drop(self, to):
        """Drops class inside project or another folder """
        target = to.inner_member_container
        if not target or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        # the name must be allways the same of the class
        kwargs['name'] = kwargs['parent'].inner_class.name
        self._access = kwargs.get('access', "public")
        self._explicit = kwargs.get('explicit', False)
        self._inline = kwargs.get('inline', False)
        self._virtual = kwargs.get('virtual', True)
        self._pure = kwargs.get('pure', False)
        self._calling = kwargs.get('calling', False)
        self._callingText = kwargs.get('callspec', False)
        self._content = kwargs.get('content', "")
        super(Destructor, self).__init__(**kwargs)
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(Destructor, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['access'] = self._access
        kwargs['explicit'] = self._explicit
        kwargs['inline'] = self._inline
        kwargs['virtual'] = self._virtual
        kwargs['pure'] = self._pure
        kwargs['calling'] = self._calling
        kwargs['callspec'] = self._callingText
        kwargs['content'] = self._content
        kwargs.update(super(Destructor, self).get_kwargs())
        return kwargs

    def UpdateExitCall(self, save=True):
        """Updates the call to exit method"""
        if save:
            self.SaveState()
        method = self.inner_class.exit_methods
        mask = r'\t__exit__\s*\(.*\);\n'
        if not method:
            #if there are not exit method, remove the call
            self._content = re.sub(mask, '', self._content)
            return
        method = method[0]
        #add the call
        if not re.search(mask, self._content):
            self._content = '\t__exit__();\n' + self._content

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the member method declaration"""
        i = self.inner_class
        i._ensure_access(pf, self._access)
        pf.writeln(self.declare)
        if self._inline:
            self.outer_class._inlines.append(self)

    @ctx.ContextImplementation()
    def WriteCode(self, f):
        """Write code to file"""
        self.WriteComment(f)
        f.writeln(self.implement)
        f.openbrace()
        f.writeln(self._content)
        f.closebrace()

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(Destructor, self).OnUndoRedoChanged()
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            index = book.GetPageIndex(self._pane)
            book.SetPageText(index, self.name)
            book.SetPageBitmap(index, self.GetTabBitmap())
            self._pane.m_editor.SetText(self._content)
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            pane = self._pane
            delattr(self, '_pane')
            # we are not able to commit here!
            # pane.Commit()
            pane.PreDelete()    # avoid gtk-critical
            self._paneIndex = book.GetPageIndex(pane)
            book.DeletePage(self._paneIndex)
        super(Destructor, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Destructor, self).OnUndoRedoAdd()
        if hasattr(self, '_paneIndex'):
            book = context.frame.docBook
            import pane
            p = pane.MethodPane(book, context.frame, self)
            self._pane = p
            book.InsertPage(self._paneIndex, p, self.tab_label,
                False, self.bitmap_index)
            delattr(self, '_paneIndex')

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        return rc.GetBitmap("destructor", self._access)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("destructor", self._access)

    @property
    def tree_label(self):
        """Get tree label"""
        epilog = ""
        if self._pure:
            epilog += "=0"
        prolog = ""
        if self._virtual:
            prolog = "virtual "
        if self._inline:
            prolog = prolog + "inline "
        return prolog + "~" + self._name + "()" + epilog

    @property
    def tab_label(self):
        """Get tree label"""
        epilog = ""
        if self._pure:
            epilog += "=0"
        prolog = ""
        if self._virtual:
            prolog = "virtual "
        if self._inline:
            prolog = prolog + "inline "
        return "~{self.name}()".format(self=self)

    @property
    def declare(self):
        """Get declare string"""
        epilog = ""
        if self._pure:
            epilog += "=0"
        prolog = ""
        if self._virtual:
            prolog = "virtual "
        if self._inline:
            prolog = prolog + "inline "
        return "{prolog}~{name}(){epilog};".format(prolog=prolog, name=self._name, epilog=epilog)

    @property
    def implement(self):
        """Gets tab label"""
        return "{self.scope}~{self._name}()".format(self=self)

    def ExistArgumentNamed(self, name):
        """Checks about argument existence"""
        return False





# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 23:24:30 2013

@author: mel
"""
import model
import model.decorator as ctx
import app.resources as rc


class Library(model.TComponent):
    """Implements a Library representation"""
    class_container = True
    context_container = True
    folder_container = True
    diagram_container = True
    module_container = True
    namespace_container = True
    function_container = True
    variable_container = True
    member_container = False

    def __init__(self, **kwargs):
        """Initialization"""
        super(Library, self).__init__(**kwargs)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        return super(Library, self).get_kwargs()

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the folder declaration"""
        self.WriteComment(pf)
        #we will follow and respect the visual order
        child_types = [model.Folder, model.cc.Namespace,
            model.cc.Class, model.cc.Enum, model.cc.MemberData,
            model.cc.Constructor, model.cc.MemberMethod,
            model.cc.Destructor, model.cc.Module,
            model.cc.Function, model.cc.Data]
        for type in child_types:
            if len(self[type]):
                for item in self[type]:
                    item.WriteDeclaration(pf)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("library")

    def ExistMemberDataNamed(self, name):
        """Check recursively about the existence of nested child member"""
        from MemberData import MemberData
        return self.hasChild(name=name, type=MemberData)

    def AddMemberData(self, name):
        """Add a member element"""
        from MemberData import MemberData
        member = MemberData(name, self)
        return member

    @property
    def nested_classes(self):
        """Returns the list of nested classes (including self)"""
        if type(self.parent) not in [model.Folder, model.cc.Class]:
            return []
        return self.parent.nested_classes



# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""
import wx
import model
import app.resources as rc
from ctx import theContext as context
import model.decorator as ctx
import tran
from tran import TransactionStack


class MemberMethod(model.Member):
    """Implements member method"""
    context_container = True
    argument_container = True

    # visual methods
    def draggable(self):
        """returns info about if the object can be moved"""
        return True

    @tran.TransactionalMethod('move method {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_member_container
        if not target or self.inner_class != target.inner_class or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        self._typei = kwargs['type']
        self._access = kwargs.get('access', 'public')
        self._static = kwargs.get('static', False)
        self._virtual = kwargs.get('virtual', False)
        self._pure = kwargs.get('pure', False)
        self._inline = kwargs.get('inline', False)
        self._const_method = kwargs.get('constmethod', False)
        self._content = kwargs.get('content', "")
        self._template = kwargs.get('template', None)
        self._template_types = kwargs.get('template_types', [])
        super(MemberMethod, self).__init__(**kwargs)
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(MemberMethod, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['type'] = self._typei
        kwargs['access'] = self._access
        kwargs['static'] = self._static
        kwargs['virtual'] = self._virtual
        kwargs['pure'] = self._pure
        kwargs['inline'] = self._inline
        kwargs['constmethod'] = self._const_method
        kwargs['content'] = self._content
        kwargs['template'] = self._template
        kwargs['template_types'] = self._template_types
        kwargs.update(super(MemberMethod, self).get_kwargs())
        return kwargs

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the member method declaration"""
        if self._declare:
            i = self.inner_class
            i._ensure_access(pf, self._access)
            pf.writeln(self.declare)
            if self._inline or i._template or self._template:
                self.outer_class._inlines.append(self)

    @ctx.ContextImplementation()
    def WriteCode(self, f):
        """Write code to file"""
        self.WriteComment(f)
        f.writeln(self.implement)
        f.openbrace()
        f.writeln(self._content)
        f.closebrace()

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(MemberMethod, self).OnUndoRedoChanged()
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            index = book.GetPageIndex(self._pane)
            book.SetPageText(index, self.name)
            book.SetPageBitmap(index, self.GetTabBitmap())
            if self._pane.m_editor.GetText() != self._content:
                self._pane.m_editor.SetText(self._content)
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            p = self._pane
            delattr(self, '_pane')
            # we are not able to commit here!
            # pane.Commit()
            p.PreDelete()    # avoid gtk-critical
            self._paneIndex = book.GetPageIndex(p)
            book.DeletePage(self._paneIndex)
        super(MemberMethod, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        import pane
        super(MemberMethod, self).OnUndoRedoAdd()
        if hasattr(self, '_paneIndex'):
            book = context.frame.docBook
            p = pane.MethodPane(book, context.frame, self)
            self._pane = p
            book.InsertPage(self._paneIndex, p, self.tab_label,
                False, wx.NOT_FOUND)
            book.SetPageBitmap(self._paneIndex, self.GetTabBitmap())
            delattr(self, '_paneIndex')

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        return rc.GetBitmap("method", self._access)

    @property
    def type_instance(self):
        """return the type instance"""
        return self._typei

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("method", self._access)

    @property
    def tree_label(self):
        """Get tree label"""
        if self._template:
            stype = 'template<{}> '.format(self._template)
        else:
            stype = ''
            if self._virtual:
                stype = "virtual "
            if self._inline:
                stype = "inline " + stype
        stype += str(self._typei)
        from Argument import Argument
        argl = ', '.join(arg.declare for arg in self[Argument])
        epilog = ""
        if self._pure:
            epilog += "=0"
        if self._const_method:
            epilog += " const"
        return stype.format(self._name + "(" + argl + ")") + epilog

    @property
    def call(self):
        """Call default expression"""
        from Argument import Argument
        argl = ', '.join(arg.name for arg in self[Argument])
        return "{name}({argl})".format(name=self.name, argl=argl)

    @property
    def tab_label(self):
        """Get tab label"""
        from Argument import Argument
        argl = ', '.join(arg.implement for arg in self[Argument])
        return "{name}({argl})".format(name=self.name, argl=argl)

    @property
    def implement(self):
        """Gets tab label"""
        from Argument import Argument
        if self._template:
            s = 'template<{}> '.format(self._template)
        elif self._virtual:
            s = "virtual "
        else:
            s = ''
        if self._inline:
            s = "inline " + s
        s += str(self._typei)
        argl = ', '.join(arg.implement for arg in self[Argument])
        epilog = ""
        if self._const_method:
            epilog += " const"
        untyped = '{self.scope}{self._name}({args})'.format(self=self, args=argl)
        return s.format(untyped) + epilog

    @property
    def declare(self):
        """declaration string inside class"""
        from Argument import Argument
        if self._template:
            s = 'template<{}> '.format(self._template)
        elif self._virtual:
            s = "virtual "
        else:
            s = ''
        if self._inline:
            s = "inline " + s
        s += str(self._typei)
        argl = ', '.join(arg.declare for arg in self[Argument])
        epilog = ""
        if self._pure:
            epilog += "=0"
        if self._const_method:
            epilog += " const"
        untyped = '{self._name}({args}){epilog}'.format(self=self, args=argl, epilog=epilog)
        return s.format(untyped) + ";"

    def ExistArgumentNamed(self, name):
        """Returns information about argument existence"""
        from Argument import Argument
        return self.hasChild(name=name, type=Argument)

    @property
    def template_types(self):
        """Returns the list of nested template types"""
        # _template_types are locally applied/defined and will override outer
        # template types (of course in any moment we need to alert from that).
        lt = self._template_types
        nt = super(MemberMethod, self).template_types
        lt.extend([x for x in nt if x not in self._template_types])
        return lt




"""Subclass of GitView, which is generated by wxFormBuilder."""

import wx
import wxx
import tran
import app
import app.ui.dlg
import model
import deco
import model.git
import app.resources as rc
from ctx import localcontext as context
from activity.git.ui import ui
from activity.git.ui import dlg as ui_dlg
from app.ui.view import BaseView


# Implementing GitView
class GitView(BaseView, ui.GitView):
    """View pane for git repository"""
    perspective = ''
    clipboard_handler = False

    #command ids
    _newRepoId = wx.Window.NewControlId()
    _openRepoId = wx.Window.NewControlId()
    _addItemId = wx.Window.NewControlId()
    _stageItemId = wx.Window.NewControlId()
    _commitItemId = wx.Window.NewControlId()
    _refreshRepoId = wx.Window.NewControlId()
    _addRemoteId = wx.Window.NewControlId()
    _pushRemoteId = wx.Window.NewControlId()

    def __init__(self, parent, root=None):
        """Initialize pane"""
        super(GitView, self).__init__(parent)
        self.frame = parent
        self.selected = None
        self.imglist = rc.GetBitmapImageList()
        self.m_tree.SetImageList(self.imglist)
        self.m_tree.AddRoot('Repositories')
        self._create_menus()
        self._bind_events()
        self._set_accelerators()
        # when we create a models view, we need to update elements
        if root is None:
            # reload full stack
            for wrk in context.app.workspaces:
                self.insert(wrk)
        else:
            self.insert(root)

    def insert(self, element):
        """Nested insert elements in tree"""
        if self.DoRenderAddElement(element):
            for cls in element._child:
                for k in element[cls]:
                    self.insert(k)

    def GetViewStatus(self):
        """Get working info about the view"""
        return str(self.m_tree.GetExpansionState())

    def SetViewStatus(self, status):
        """Set working info about the view"""
        try:
            l = eval(status)
            self.m_tree.SetExpansionState(l)
            return True
        except:
            return False


    @deco.classproperty
    def name(cls):
        """returns the name of this view"""
        return 'Git'

    @classmethod
    def bitmap_index(cls):
        """return the bitmap index"""
        return rc.GetBitmapIndex('git')

    @classmethod
    def bitmap(cls):
        """return the bitmap"""
        return rc.GetBitmap('git')

    def _create_menus(self):
        """Create a custom menu entry in main menu"""
        self._menu = wx.Menu()

        # New repository
        self._newRepo = wx.MenuItem(self._menu, self._newRepoId, u"New repository",
            u"create new git repository", wx.ITEM_NORMAL)
        self._newRepo.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_NEW, wx.ART_MENU))
        self._menu.AppendItem(self._newRepo)

        #Open repository
        self._openRepo = wx.MenuItem(self._menu, self._openRepoId, u"Open repository",
            u"open existing repository", wx.ITEM_NORMAL)
        self._openRepo.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_MENU))
        self._menu.AppendItem(self._openRepo)

        self._menu.AppendSeparator()
        #Add remote
        self._addRemote = wx.MenuItem(self._menu, self._addRemoteId, u"Add remote",
            u"add remote repository", wx.ITEM_NORMAL)
        self._addRemote.SetBitmap(wx.Bitmap(u"app/res/git_remote.xpm", wx.BITMAP_TYPE_ANY))
        self._menu.AppendItem(self._addRemote)

        #push remote
        self._pushRemote = wx.MenuItem(self._menu, self._pushRemoteId, u"Push remote",
            u"push to remote repository", wx.ITEM_NORMAL)
        self._pushRemote.SetBitmap(wx.Bitmap(u"app/res/git_push.xpm", wx.BITMAP_TYPE_ANY))
        self._menu.AppendItem(self._pushRemote)

        self._menu.AppendSeparator()

        #Add item to repo
        self._addItem = wx.MenuItem(self._menu, self._addItemId, u"Add to repository",
            u"add item to repository control", wx.ITEM_NORMAL)
        self._menu.AppendItem(self._addItem)

        #Add item to repo
        self._stageItem = wx.MenuItem(self._menu, self._stageItemId, u"Stage item",
            u"stage item to repository control", wx.ITEM_NORMAL)
        self._menu.AppendItem(self._stageItem)

        #Commit repo
        self._commitItem = wx.MenuItem(self._menu, self._commitItemId, u"Commit repository",
            u"commit changes to repository", wx.ITEM_NORMAL)
        self._menu.AppendItem(self._commitItem)

        self._refreshRepo = wx.MenuItem(self._menu, self._refreshRepoId, u"Refresh",
            u"update repository status", wx.ITEM_NORMAL)
        self._refreshRepo.SetBitmap(rc.GetBitmap('reload'))
        self._menu.AppendItem(self._refreshRepo)

        self.RegisterMenu('Git', self._menu)

    def _bind_events(self):
        """Binds events"""
        # Tree events
        self.m_tree.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnTreeSelChanged)
        self.m_tree.Bind(wx.EVT_RIGHT_DOWN, self.OnTreeMenu)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateDelete, id=app.ID_DELETE)
        self.BindSpecial(wx.EVT_MENU, self.OnDelete, id=app.ID_DELETE)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateNewRepository, id=self._newRepoId)
        self.BindSpecial(wx.EVT_MENU, self.OnNewRepository, id=self._newRepoId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateOpenExistingRepository, id=self._openRepoId)
        self.BindSpecial(wx.EVT_MENU, self.OnOpenExistingRepository, id=self._openRepoId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddRemote, id=self._addRemoteId)
        self.BindSpecial(wx.EVT_MENU, self.OnAddRemote, id=self._addRemoteId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdatePushRemote, id=self._pushRemoteId)
        self.BindSpecial(wx.EVT_MENU, self.OnPushRemote, id=self._pushRemoteId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddItemToRepository, id=self._addItem.GetId())
        self.BindSpecial(wx.EVT_MENU, self.OnAddItemToRepository, id=self._addItemId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateStageItem, id=self._stageItem.GetId())
        self.BindSpecial(wx.EVT_MENU, self.OnStageItem, id=self._stageItemId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateCommitRepository, id=self._commitItem.GetId())
        self.BindSpecial(wx.EVT_MENU, self.OnCommitRepository, id=self._commitItemId)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateRefreshStatus, id=self._refreshRepo.GetId())
        self.BindSpecial(wx.EVT_MENU, self.OnRefreshStatus, id=self._refreshRepoId)

        super(GitView, self)._bind_events()

    def _set_accelerators(self):
        """Set the accelerator table"""
        # ctrl_alt = wx.ACCEL_CTRL + wx.ACCEL_ALT
        # ctrl_shf = wx.ACCEL_CTRL + wx.ACCEL_SHIFT
        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_LEFT, BaseView._leftKeyId)
        ])
        self.SetAcceleratorTable(aTable)

    def DoRenderAddElement(self, obj):
        """Inserts element in tree"""
        # model tree
        treeOrder = [model.Workspace, model.git.GitRepo, model.git.GitRemotes,
            model.git.GitRemote, model.git.GitDir, model.git.GitFile]
        if type(obj) not in treeOrder:
            return False
        ti = treeOrder.index(type(obj))
        #check parent
        if self.m_tree.HoldsObject(obj.parent):
            p = obj.parent
        else:
            p = self.m_tree.GetRootItem()
        if obj.parent is not None:
            # find some major friend item with the same class
            tribal = obj.parent[type(obj)]
            index = tribal.index(obj)
            pre = None
            while index > 0 and pre is None:
                index = index - 1
                candidate = tribal[index]
                if not self.m_tree.HoldsObject(candidate):
                    continue
                pre = candidate
            if pre is not None:
                self.m_tree.InsertItem(p, pre, obj.label,
                    obj.bitmap_index, obj.bitmap_index, obj)
                if type(obj) is model.cc.Constructor:
                    self.m_tree.SetItemBold(obj, bold=obj.IsPreferred())
                elif type(obj) in [model.py.Module, model.py.Package]:
                    self.m_tree.SetItemBold(obj, bold=obj._entry)
                return True
        itemCount = 0
        citem, cookie = self.m_tree.GetFirstChild(p)
        if type(citem) is wx.TreeItemId:
            citem = self.m_tree.__fer__(citem)
        if type(citem) in treeOrder:
            if ti <= treeOrder.index(type(citem)):
                self.m_tree.PrependItem(p, obj.label,
                     obj.bitmap_index, obj.bitmap_index, obj)
                return True
        while type(citem) is not wx.TreeItemId or citem.IsOk():
            itemCount = itemCount + 1
            citem, cookie = self.m_tree.GetNextChild(p, cookie)
            if type(citem) not in treeOrder:
                continue
            if ti <= treeOrder.index(type(citem)):
                self.m_tree.InsertItemBefore(p, itemCount,
                     obj.label,
                     obj.bitmap_index, obj.bitmap_index, data=obj)
                return True
        #Ok, do apppend
        self.m_tree.AppendItem(p, obj.label,
            obj.bitmap_index, obj.bitmap_index, obj)
        return True

    def DoRenderRemoveElement(self, obj):
        """Do remove element in tree"""
        if self.m_tree.HoldsObject(obj):
            self.m_tree.Delete(obj)

    def UpdateElement(self, obj):
        """Update the tree label for a object"""
        if not self.m_tree.HoldsObject(obj):
            return
        self.m_tree.SetItemText(obj, obj.label)
        self.m_tree.SetItemImage(obj,
            obj.bitmap_index, wx.TreeItemIcon_Normal)
        if hasattr(obj, 'bitmap_open_index'):
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
        self.m_tree.SetItemImage(obj,
            obj.bitmap_index, wx.TreeItemIcon_Selected)

    def OnTreeSelChanged(self, event):
        """Handle select changed"""
        self.selected = self.m_tree.GetSelection()

    def OnUpdateNewRepository(self, event):
        """Update new repository command"""
        event.Enable(bool(self.selected and type(self.selected) is model.Workspace))

    @tran.TransactionalMethod('new git repo {}')
    @wxx.CreationDialog(ui_dlg.GitRepoDialog, model.git.GitRepo)
    def OnNewRepository(self, event):
        """New repository command"""
        return (context.frame, self.selected.inner_repository_container)

    def OnUpdateOpenExistingRepository(self, event):
        """Update open existing repository"""
        event.Enable(bool(self.selected and type(self.selected) is model.Workspace))

    @tran.TransactionalMethod('add git repo {}')
    @wxx.CreationDialog(ui_dlg.OpenGitRepoDialog, model.git.GitRepo)
    def OnOpenExistingRepository(self, event):
        """Handle add folder command"""
        return (context.frame, self.selected.inner_repository_container)

    def OnUpdateAddRemote(self, event):
        """Update add remote repo"""
        event.Enable(bool(self.selected and type(self.selected) is not model.Workspace
            and self.selected.repo))

    @tran.TransactionalMethod('add remote repo {}')
    @wxx.CreationDialog(ui_dlg.GitRemoteDialog, model.git.GitRemote)
    def OnAddRemote(self, event):
        """Handle add remote repo"""
        rl = self.selected.repo(model.git.GitRemotes)
        return (context.frame, rl[0])

    def OnUpdatePushRemote(self, event):
        """Update add remote repo"""
        event.Enable(bool(self.selected and type(self.selected) is model.git.GitRemote))

    def OnPushRemote(self, event):
        """Handle add remote repo"""
        _repo = self.selected.repo._repo
        remote = _repo.remote(self.selected.name)
        dialog = ui_dlg.ProgressDialog(self.frame)
        dialog.Show()
        wx.YieldIfNeeded()
        try:
            remote.push(progress=dialog.link())
            dialog.update('--', 0, message='DONE')
        except:
            dialog.update('--', 100, message='FAILED')
            pass
        dialog.Finished()
        return
        import git
        g = git.cmd.Git(self.selected.repo._uri)
        g.push('--set-upstream', self.selected.name, 'master')
        return
        _repo = self.selected.repo._repo
        ref = _repo.remote(self.selected.name)
        try:
            dialog = ui_dlg.ProgressDialog(self, self.frame)
            ui_dlg.Show()
            if self.selected.password:
                ref.push(refspec='refs/heads/master:refs/heads/master', password=self.selected.password,
                    progress=dialog)
            else:
                ref.push(refspec='refs/heads/master:refs/heads/master', progress=dialog)
        except Exception, e:
            wx.MessageBox("Push to repository {name} failed: {error}.".format(
                name=self.selected.name, error=e), "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
        dialog.Finished()

    def TreeLeftKey(self, event):
        """If the selected node is expanded, simply collapse it.
        If not, navigate through parent"""
        if not self.selected:
            return
        if self.m_tree.IsExpanded(self.selected):
            self.m_tree.Collapse(self.selected)
        else:
            parent = self.selected.parent
            if self.m_tree.HoldsObject(parent):
                self.m_tree.SelectItem(parent)

    def OnTreeMenu(self, event):
        """Handles context tree popup menu"""
        (item, where) = self.m_tree.HitTest(event.GetPosition())
        if item is None:
            return
        self.m_tree.SelectItem(item)
        if self.selected is None:
            return
        menu = app.ui.clone_mnu(self._menu, enabled=True, notitle=True)
        if menu is None:
            return
        self.PopupMenu(menu, event.GetPosition())
        menu.Destroy()

    def OnDelete(self, event):
        """delete element"""
        obj = self.selected
        aliasMap = {
            model.git.GitRepo: "git repository {}",
            model.git.GitFile: "git file {}",
            model.git.GitDir: "git dir {}",
            model.git.GitRemote: "git remote {}"}
        tran.TransactionStack.DoBeginTransaction("delete " +
             aliasMap[type(obj)].format(obj._name))
        obj.Delete()
        tran.TransactionStack.DoCommit()

    def OnUpdateDelete(self, event):
        """Update delete element"""
        obj = self.selected
        event.Enable((not obj is None) and type(obj) is not model.Workspace
            and obj.can_delete)

    def OnUpdateCommitRepository(self, event):
        """Update commit to repo"""
        obj = self.selected
        event.Enable(bool(obj and type(obj) is model.git.GitRepo
        and obj.stage_size > 0))

    def OnCommitRepository(self, event):
        """Process a commit"""
        dialog = ui_dlg.CommitGitDialog(context.frame)
        if dialog.ShowModal() == wx.ID_OK:
            self.selected.commit(dialog._message)

    def OnUpdateRefreshStatus(self, event):
        """Update refresh status"""
        obj = self.selected
        event.Enable((not obj is None) and type(obj) is not model.Workspace)

    @tran.TransactionalMethod('refresh git status')
    def OnRefreshStatus(self, event):
        """Refresh repository status"""
        working = app.ui.dlg.WorkingDialog(context.frame)
        working.Show(True)
        wx.YieldIfNeeded()
        self.selected.update_status()
        working.Close()
        return True

    def OnUpdateAddItemToRepository(self, event):
        """Update the operation of adding elements to repo"""
        obj = self.selected
        if not obj:
            event.Enable(False)
            return
        if type(obj) is model.git.GitFile:
            if obj._status == 'file':
                event.Enable(True)
            else:
                event.Enable(False)
            return
        #ok, buscamos un hijo
        fn = lambda x: (x._status == 'file')
        if obj(model.git.GitFile, filter=fn):
            event.Enable(True)
        else:
            event.Enable(False)

    def OnAddItemToRepository(self, event):
        """Add element to track"""
        obj = self.selected
        repo = obj.repo
        if type(obj) is model.git.GitFile:
            repo.add(obj)
            return
        fn = lambda x: (x._status == 'file')
        for item in obj(model.git.GitFile, filter=fn):
            repo.add(item)

    def OnUpdateStageItem(self, event):
        """Update the operation of adding elements to repo"""
        obj = self.selected
        if not obj:
            event.Enable(False)
            return
        # para hacer stage es suficiente que el elemento
        # seleccionado o un hijo sea un GitFile con status modified
        if type(obj) is model.git.GitFile:
            if obj._status == 'git_file_modified':
                event.Enable(True)
            else:
                event.Enable(False)
            return
        # ok, buscamos un hijo
        fn = lambda x: (x._status == 'git_file_modified')
        if obj(model.git.GitFile, filter=fn):
            event.Enable(True)
        else:
            event.Enable(False)

    def OnStageItem(self, event):
        """Add element to track"""
        obj = self.selected
        repo = obj.repo
        if type(obj) is model.git.GitFile:
            repo.add(obj)
            return
        fn = lambda x: (x._status == 'git_file_modified')
        for item in obj(model.git.GitFile, filter=fn):
            repo.add(item)

    def OnUpdateEditProperties(self, event):
        """"""
        event.Enable(False)

    def OnEditProperties(self, event):
        """"""
        pass


# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 23:24:30 2013

@author: mel
"""

import os
import sys
import traceback
import wx
import model
import analytic
from model.writer import writer
from Import import Import
from ImportsFolder import ImportsFolder
import app.resources as rc
import tran


class Module(model.TComponent):
    """Implements a Module representation"""
    class_container = True
    folder_container = True
    diagram_container = True
    namespace_container = False
    function_container = True
    variable_container = True
    enum_container = False
    import_container = True

    def __init__(self, **kwargs):
        """Initialization"""
        self._lastSrcTime = None
        self._lastHdrTime = None
        self._entry = False
        self._content = kwargs.get('content', "")
        self._file_obj = None  # vincle with file object
        super(Module, self).__init__(**kwargs)
        self.ExportPythonCodeFiles()

    def set_file(self, file_obj):
        """Sets the vincle with file object"""
        self._file_obj = file_obj

    def Delete(self):
        """delete the object"""
        if hasattr(self, '_file_obj') and self._file_obj:
            self._file_obj.Delete()
        super(Module, self).Delete()

    @property
    def label(self):
        """Get tree label"""
        return '{self._name}'.format(self=self)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['content'] = self._content
        kwargs.update(super(Module, self).get_kwargs())
        return kwargs

    @property
    def key_file(self):
        """This method returns the key file. For modules, is
        the implementation file. For packages, is the __init__ file."""
        fname = os.path.join(self.dir, '{self._name}.py'.format(self=self))
        return os.path.realpath(fname)

    @property
    def dir(self):
        """return the container dir"""
        return self.parent.dir

    @property
    def path_name(self):
        """Get the full path"""
        if self.parent.inner_module is not None:
            return '{self.parent.inner_module.path_name}.{self._name}'.format(self=self)
        return self._name

    def OnUndoRedoRemoving(self):
        """Handle OnUndoRedoRemoving, prevent generating files"""
        stack = tran.TransactionStack
        method = self.ExportPythonCodeFiles.inner
        if method not in stack.delayedCallsFiltered:
            stack.delayedCallsFiltered.append(method)
        super(Module, self).OnUndoRedoRemoving()

    def ExportPythonFolderCodeFiles(self, fw, folder, force, logger=None):
        """does python export for folders"""
        for subfolder in folder[model.Folder]:
            self.ExportPythonFolderCodeFiles(fw, subfolder, force, logger)

        for obj in folder[ImportsFolder]:
            obj.ExportPythonCode(fw)

        for obj in folder[Import]:
            obj.ExportPythonCode(fw)

        from Class import Class
        for obj in folder[Class]:
            obj.ExportPythonCode(fw)

        from Function import Function
        for obj in folder[Function]:
            obj.ExportPythonCode(fw)

        from Data import Data
        for obj in folder[Data]:
            obj.ExportPythonCode(fw)

    @property
    def abs_path(self):
        """return the absolute path"""
        return os.path.join(self.dir, '{self._name}.py'.format(self=self))

    @tran.DelayedMethod
    def ExportPythonCodeFiles(self, force=False, logger=None):
        """does source generation"""
        #The mission of this method is to generate the required code
        #for the python
        fname = self.abs_path
        logger and logger.AppendReportLine('exporting module {}'.format(self._name), wx.ICON_INFORMATION)
        try:
            f = open(fname, 'w')
            fw = writer.for_file(f, 'python')

            #first thing: generate module comment
            fw.writeln("# -*- coding: utf-8 -*-")
            fw.writecomment(self._note)
            fw.writenl()

            #we have some problem here with the order the code is generated
            #for solving that, we use nested folders
            for folder in self[model.Folder]:
                self.ExportPythonFolderCodeFiles(fw, folder, force, logger)

            #declare the orderer collections of things
            #that would be exported

            #fw.writeln("#imports")
            for obj in self[ImportsFolder]:
                obj.ExportPythonCode(fw)
            for obj in self[Import]:
                obj.ExportPythonCode(fw)

            from Class import Class
            for obj in self[Class]:
                obj.ExportPythonCode(fw)

            from Function import Function
            for obj in self[Function]:
                obj.ExportPythonCode(fw)

            from Data import Data
            for obj in self[Data]:
                obj.ExportPythonCode(fw)

            #At last, write raw code
            fw.writenl(1)
            fw.writeln("#raw code")
            fw.writeblock(self._content)

            f.close()

        except Exception as inst:
            traceback.print_exc(file=sys.stdout)
            print type(inst)     # the exception instance
            print inst.args      # arguments stored in .args
            print inst
            if logger:
                logger.AppendReportLine('failed opening {}'.format(fname), wx.NOT_FOUND)
            return False

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        return rc.GetBitmap("py_module")

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("py_module")

    @property
    def inner_class(self):
        """Get the innermost class container"""
        return None

    def analize(self):
        """Do an analisis of the raw content and extracts
        whatever its possible to extract"""
        self.DeleteChilds()
        s = analytic.pyparse(self, self._content)
        self._content = s.inline()
        return s._status

    @property
    def locals(self):
        """This method returns all the top elements defined
        in the module"""
        f = lambda x: x.inner_module == self
        raw = self(model.py.Class, model.py.Data, model.py.Function, filter=f)
        return dict([(e._name, e) for e in raw])

    def _local_import(self, name):
        """This method finds for a module or package (in this order)
        matching the name of the import, inside the same directory of
        the module. Please take consideration of that
        the name must be simple, without dots. Returns the module or package
        or None if not found"""
        container = self.inner_package
        f = lambda x: x != self and x.inner_package == container
        for x in self(model.py.Module, filter=f, cut=True):
            if x._name == name:
                return x
        f = lambda x: x.parent.inner_package == container
        for x in self(model.py.Package, filter=f, cut=True):
            if x._name == name:
                return x
        return None

    def _global_import(self, name):
        """This method finds for a module or package (in this order)
        matching the name of the import, inside the root directory of
        the project. Please take consideration of that
        the name must be simple, without dots. Returns the module or package
        or None if not found"""
        f = lambda x: x != self and x.inner_package is None
        for x in self(model.py.Module, filter=f, cut=True):
            if x._name == name:
                return x
        f = lambda x: x.parent.inner_package is None
        for x in self(model.py.Package, filter=f, cut=True):
            if x._name == name:
                return x
        return None

    def _import(self, name, root=True):
        """Get the import (if possible)"""
        sections = name.split('.')
        if not len(sections):
            return None
        base = sections[0]
        del sections[0]
        base = self._local_import(base) or self._global_import(base)
        if base is None:
            #this import is external
            import ostools
            return ostools.import_dir(name)
        for next in sections:
            base = base._local_import(next)
            if base is None:
                return None
        return base

    @property
    def globals(self):
        """This method returns all the globals 'visible' inside the module"""
        # basicly we iterate through the imports
        f = lambda x: x.inner_module == self
        imports = self(model.py.Import, filter=f)
        #
        d = {}
        for imp in imports:
            # buscamos el modulo o fuente que se importa.
            m = self._import(imp._name)
            if m is None:
                #pending log failed import
                continue
            # m puede ser un Module, un Package o bien una lista de simbolos
            # procedentes de una importacion nativa
            if imp._from is not None:
                #en este caso lo que se importa es un simbolo
                #buscamos el simbolo
                if type(m) is list:
                    if imp._from not in m:
                        # pending log misssing symbol
                        continue
                    symbol = m[imp._from]
                elif type(m) is model.py.Package:
                    symbol = m._local_import(imp._from)
                    if imp._from not in m:
                        # pending log misssing symbol
                        continue
                    symbol = m[imp._from]
                elif type(m) is model.py.Module:
                    l = m.locals()
                    l.extend(m.globals())
                    if imp._from not in l:
                        #pending log missing symbol
                        continue
                    symbol = l[imp._from]
            else:
                symbol = m
            if imp._as:
                d[imp._as] = symbol
            else:
                d[imp._name] = symbol
        return d




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:31:28 2013

@author: mel
"""

import model
import tran
import app.resources as rc
from tran import TransactionStack


class Argument(model.TComponent):
    """Implements argument representation"""

    context_container = True

    #visual methods
    @tran.TransactionalMethod('move argument {}')
    def drop(self, to):
        """drop this elemento to another place"""
        target = to.inner_argument_container
        if not target or to.project != self.project:
            return False  # avoid move arguments between projects
        index = 0  # trick for insert as first child
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        self._typei = kwargs['type']
        self._default = kwargs.get('default', '')
        super(Argument, self).__init__(**kwargs)
        container = self.outer_class or self.outer_module
        container._lastSrcTime = None
        container._lastHdrTime = None
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(Argument, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['type'] = self._typei
        kwargs['default'] = self._default
        kwargs.update(super(Argument, self).get_kwargs())
        return kwargs

    def OnUndoRedoChanged(self):
        """Update from app"""
        self.parent.OnUndoRedoChanged()
        super(Argument, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    def OnUndoRedoRemoving(self, root=True):
        """Do required actions for removing"""
        super(Argument, self).OnUndoRedoRemoving(root)
        self.parent.OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        self.parent.OnUndoRedoChanged()
        super(Argument, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('member')

    @property
    def tree_label(self):
        """Get tree label"""
        init = ''
        if len(self._default) > 0:
            init += "=" + self._default
        return (str(self._typei).format(self._name) + init).encode('ascii', 'ignore')

    @property
    def implement(self):
        """return the label whitout default value"""
        return str(self._typei).format(self._name).encode('ascii', 'ignore')

    @property
    def declare(self):
        """return the label whitout default value"""
        init = ''
        if len(self._default) > 0:
            init += "=" + self._default
        return (str(self._typei).format(self._name) + init).encode('ascii', 'ignore')




"""Subclass of NewEnumItem, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui


# Implementing NewEnumItem
class EnumItemDialog(ui.NewEnumItem):
    """
    This dialog allows to edit enum item.
    You can set an item explicit value or
    leave it blank for default value.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, item=None):
        """Initialise dialog"""
        super(EnumItemDialog, self).__init__(parent)
        if item is not None:
            self.m_textCtrl56.SetValue(item[0])
            if len(item[1].strip()) > 0:
                self.m_comboBox6.Append(item[1])
            i = self.m_comboBox6.FindString(item[1].strip())
            self.m_comboBox6.SetSelection(i)
        self.origin = item
        self.parent = parent

    #Validate dialog
    def Validate(self):
        """Get values and validate"""
        self._label = self.m_textCtrl56.GetValue().strip()
        self._value = self.m_comboBox6.GetValue().strip()
        if not len(self._label):
            wx.MessageBox("enum item label must be not empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._label) is None:
            wx.MessageBox("enum item label contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if len(self._value) and not re.match("^[0-9]*$", self._value):
            y = wx.MessageBox("enum item value is not a number. You are sure?", "Warning",
                wx.YES_NO | wx.CENTER | wx.ICON_ERROR, self)
            if y != wx.YES:
                return False
        if not self.origin or self.origin[0] != self._label:
            if self.parent.m_listCtrl5.FindItem(wx.NOT_FOUND, self._label) != wx.NOT_FOUND:
                wx.MessageBox("the item {} already exists".format(self._label), "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        return True

    # Handlers for NewEnumItem events.
    def OnCancel(self, event):
        """Process cancel"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Process ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
import wx
from  ctx import theContext as context
from ToolLogExplorerPane import ToolLogExplorerPane
import app.resources as rc


class log_explorer(wx.PyEvtHandler):
    """Class for providing log explorer"""
    instance = None

    def __init__(self):
        """Initialize log explorer"""
        super(log_explorer, self).__init__()
        #create the menus
        self._menuid = context.frame.new_tool_id()
        self._imenu = wx.MenuItem(context.frame.menuTools, self._menuid, u"Log explorer",
            u"show current transaction log", wx.ITEM_NORMAL)
        context.frame.AddToolsMenu('Log explorer', self._imenu)
        #bind the menu handlers
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateLogExplorer, id=self._menuid)
        self.Bind(wx.EVT_MENU, self.OnLogExplorer, id=self._menuid)

    @classmethod
    def load(cls):
        """Setup tool for the environment"""
        return log_explorer()

    def OnUpdateLogExplorer(self, event):
        """Handle the update"""
        event.Enable(True)

    def OnLogExplorer(self, event):
        """Handle the command"""
        # create a pane
        frame = context.frame
        p = ToolLogExplorerPane(frame.docBook)
        frame.docBook.AddPage(p, "Log explorer", True, rc.GetBitmapIndex("mini_logo"))




# -*- coding: utf-8 -*-
"""
Created on Mon Dec 16 08:28:25 2013

@author: mel
"""


import model
import app.resources as rc
import tran
from tran import TransactionStack


class Data(model.TComponent):
    """Implements data"""
    context_container = True

    # visual methods
    @tran.TransactionalMethod('move variable {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_variable_container
        if not target:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        "Initialization"
        self._typei = kwargs['type']
        self._static = kwargs.get('static', False)
        self._volatile = kwargs.get('volatile', False)
        self._default = kwargs.get('default', "")
        super(Data, self).__init__(**kwargs)
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(Data, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['type'] = self._typei
        kwargs['access'] = self._access
        kwargs['static'] = self._static
        kwargs['volatile'] = self._volatile
        kwargs['default'] = self._default
        kwargs.update(super(Data, self).get_kwargs())
        return kwargs

    def WriteCode(self, pf):
        """wtite data definition"""
        pf.writeln("{};".format(self.label))

    def GetInitializer(self):
        """Return the initializer sequence"""
        if len(self._default) > 0:
            return self._default
        return self._name

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("data")

    def GetStaticDefinition(self):
        """Return global, static definition"""
        if not self._static:
            return ''
        stype = str(self._typei)
        if self._volatile:
            stype = "volatile " + stype
        decl = stype.format(self.scoped_name)
        return '{decl} = {value}'.format(decl, self._default)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Data, self).OnUndoRedoAdd()

    def OnUndoRedoChanged(self):
        """Make changes in the model as result of change"""
        super(Data, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    @property
    def label(self):
        """Get tree label"""
        s = ''
        s = str(self._typei)
        if self._volatile:
            s = "volatile " + s
        if self._static:
            s = "static " + s
        epi = ""
        return s.format(self._name) + " " + epi

    @property
    def scope(self):
        """Returns the scope string ie <namespace>::<namespace>::...::
           Returns empty string instead of '::' """
        return self.parent.scope




# -*- coding: utf-8 -*-
"""
Created on Mon Dec 16 08:28:25 2013

@author: mel
"""


import model
import app.resources as rc
import tran


class Data(model.TComponent):
    """Implements data"""
    context_container = True

    # visual methods
    @tran.TransactionalMethod('move python variable {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_variable_container
        if not target:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        "Initialization"
        self._value = kwargs.get('value', "'None'")
        super(Data, self).__init__(**kwargs)
        k = self.inner_module or self.inner_package
        if k:
            k.ExportPythonCodeFiles()

    def Delete(self):
        """Handle delete"""
        k = self.inner_module or self.inner_package
        super(Data, self).Delete()
        if k:
            k.ExportPythonCodeFiles()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['value'] = self._value
        kwargs.update(super(Data, self).get_kwargs())
        return kwargs

    def WriteCode(self, pf):
        """wtite data definition"""
        pass

    def GetInitializer(self):
        """Return the initializer sequence"""
        if len(self._value) > 0:
            return self._value
        return self._name

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("py_variable")

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Data, self).OnUndoRedoAdd()

    def OnUndoRedoChanged(self):
        """Make changes in the model as result of change"""
        super(Data, self).OnUndoRedoChanged()

    def ExportPythonCode(self, wf):
        """Write code"""
        if len(self._value):
            wf.writeln('{self._name} = {self._value}'.format(self=self))
        else:
            wf.writeln('{self._name} = None'.format(self=self))




"""Subclass of Expressions, which is generated by wxFormBuilder."""

from activity.files.ui import ui as ui


# Implementing Expressions
class Expressions(ui.Expressions):
    """Pane showing user expressions to be evaluated"""

    def __init__(self, parent, handler):
        """Initialize pane"""
        self._handler = handler
        super(Expressions, self).__init__(parent)




"""Subclass of FullScreen, which is generated by wxFormBuilder."""

import wx
import app
from ctx import localcontext as context


# Implementing FullScreen
class FullScreen(app.FullScreen):
    """Implements fullscreen pane"""

    def __init__(self, pane):
        """Initialize pane"""
        super(FullScreen, self).__init__(None)
        self._frame = context.frame
        #context.SetFrame(self)
        self._pane = pane
        self._index = pane._notebook.GetPageIndex(pane)
        pane._notebook.RemovePage(self._index)
        sizer = wx.FlexGridSizer(1, 1, 0, 0)
        sizer.AddGrowableCol(0)
        sizer.AddGrowableRow(0)
        sizer.SetFlexibleDirection(wx.BOTH)
        sizer.SetNonFlexibleGrowMode(wx.FLEX_GROWMODE_SPECIFIED)
        pane.Reparent(self)
        sizer.Add(pane, 1, wx.EXPAND | wx.ALL, 0)
        self.SetSizer(sizer)
        self.Layout()
        self.Centre(wx.BOTH)
        self._frame.Hide()
        self.ShowFullScreen(True)
        wx.GetApp().SetTopWindow(self)

    # Handlers for FullScreen events.
    def LeaveFullScreen(self):
        """Toggle fullscreen"""
        self._pane.Reparent(None)
        obj = self._pane._object
        text = obj.tab_label
        self._pane._notebook.InsertPage(self._index, self._pane, text, True, obj.bitmap_index)
        #self._pane._notebook.SetPageBitmap(self._index, obj.bitmap_index)
        #context.SetFrame(self._frame)
        self._frame.Show()
        wx.GetApp().SetTopWindow(context.frame)



"""Subclass of NewPyMethod, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewPyMethod
class PyFunctionDialog(ui.NewPyMethod):
    """
    This dialog allows to setup non-member methods.
    This kind of elements may be declared inside
    a module.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(PyFunctionDialog, self).__init__(parent)
        self.container = container
        self.m_textCtrl5.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_function"))
        self.SetIcon(icon)

    def Validate(self):
        """Validation"""
        self._name = self.m_textCtrl5.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Function name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("Function name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._implement = self.m_checkBox86.IsChecked()
        self._note = self.m_richText1.GetValue()
        return True

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self.container
        kwargs['name'] = self._name
        kwargs['implement'] = self._implement
        kwargs['note'] = self._note
        return kwargs

    def CopyAttributes(self, method):
        """Copy attributes to method"""
        method._name = self._name
        method._implement = method._implement
        method.note = self._note

    def SetAttributes(self, method):
        """Setup attributes for editing already method"""
        self.m_textCtrl5.SetValue(method._name)
        self.m_checkBox86.SetValue(method._implement)
        self.m_richText1.SetValue(method.note)
        self.SetTitle("Edit method")

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





"""Subclass of TargetsView, which is generated by wxFormBuilder."""

import wx
import app
import deco
import app.resources as rc
from app.ui.view import BaseView
from activity.targets.ui import ui
from app.ui.view import BaseView


# Implementing TargetsView
class TargetsView(BaseView, ui.TargetsView):
    """Implements target view"""
    perspective = ''

    def __init__(self, parent):
        """Initialize target"""
        super(TargetsView, self).__init__(parent)

    @deco.classproperty
    def name(cls):
        """returns the name of this view"""
        return 'Targets'

    @classmethod
    def bitmap_index(cls):
        """return the bitmap index"""
        return rc.GetBitmapIndex('target')

    @classmethod
    def bitmap(cls):
        """return the bitmap"""
        return rc.GetBitmap('target')

    # Handlers for TargetsView events.
    def OnTreeOpenFile(self, event):
        """Process open file event"""
        pass

    def OnTreeSelChanged(self, event):
        """Process on tree change selection"""
        # TODO: Implement OnTreeSelChanged
        pass




import os
import wxversion

# remove conficts beetween IM and wx
if 'GTK_IM_MODULE' in os.environ:
    del os.environ['GTK_IM_MODULE']

wxversion.select('3.0')


import app
appInstance = app.proCxx(0)
appInstance.MainLoop()



# -*- coding: utf-8 -*-
import wx
import wxx
import tran
import model
from app.ui import dlg
from activity.models.ui import dlg as mdlg

edit_dict = {
        model.Workspace: (dlg.WorkspaceDialog, 'edit workspace {}'),
        model.cc.Namespace: (mdlg.NamespaceDialog, 'edit namespace {}'),
        model.cc.Type: (mdlg.TypeDialog, 'edit type {}'),
        model.cc.Class: (mdlg.ClassDialog, 'edit c++ class {}'),
        model.py.Class: (mdlg.PyClassDialog, 'edit python class {}'),
        model.cc.Argument: (mdlg.ArgumentDialog, 'edit c++ method argument {}'),
        model.py.Argument: (mdlg.PyArgumentDialog, 'edit python method argument {}'),
        model.Folder: (dlg.FolderDialog, 'edit folder {}'),
        model.ClassDiagram: (mdlg.ClassDiagramDialog, 'edit class diagram {}'),
        model.cc.Inheritance: (mdlg.InheritanceDialog, 'edit c++ inheritance {}'),
        model.py.Inheritance: (mdlg.PyInheritanceDialog, 'edit python inheritance {}'),
        model.cc.MemberData: (mdlg.MemberDialog, 'edit c++ class member {}'),
        model.py.MemberData: (mdlg.PyMemberDialog, 'edit python class member {}'),
        model.cc.Data: (mdlg.VariableDialog, 'edit c++ variable {}'),
        model.py.Data: (mdlg.PyVariableDialog, 'edit python variable {}'),
        model.cc.Constructor: (mdlg.ConstructorDialog, 'edit c++ {} constructor'),
        model.py.InitMethod: (mdlg.PyMemberMethodDialog, 'edit python __init__ method'),
        model.cc.MemberMethod: (mdlg.MemberMethodDialog, 'edit c++ method {}'),
        model.py.MemberMethod: (mdlg.PyMemberMethodDialog, 'edit python method {}'),
        model.cc.IsClassMethod: (mdlg.MemberMethodDialog, 'edit c++ method {}'),
        model.cc.Destructor: (mdlg.DestructorDialog, 'edit c++ destructor {}'),
        model.cc.Module: (mdlg.ModuleDialog, 'edit c++ module "{}"'),
        model.cc.Function: (mdlg.FunctionDialog, 'edit c++ function {}'),
        model.py.Function: (mdlg.PyFunctionDialog, 'edit python function {}'),
        model.Note: (mdlg.NoteDialog, 'edit note'),
        model.cc.Enum: (mdlg.EnumDialog, 'edit c++ enum {}'),
        model.Project: (mdlg.ProjectDialog, 'edit project {}'),
        model.cc.RelationFrom: (mdlg.RelationDialog, 'edit relation'),
        model.cc.RelationTo: (mdlg.RelationDialog, 'edit relation'),
        model.py.Module: (mdlg.PyModuleDialog, 'edit python module {}'),
        model.py.Import: (mdlg.PyImportDialog, 'edit python import {}'),
        model.py.Decorator: (mdlg.PyDecoratorDialog, 'edit python decorator {}'),
        model.cc.pyBoostModule: (mdlg.BoostPythonModuleDialog, 'edit boost.python module {}'),
    }


def edit(parent, obj):
    """Edit the object"""
    t = type(obj)
    if t in edit_dict:
        v = edit_dict[t]

        @tran.TransactionalMethod(v[1])
        @wxx.EditionDialog(v[0])
        def editDialog(parent, obj):
            """Handle type edition"""
            return (parent, obj)

        editDialog(parent, obj)
        return True
    return False


def append_menuitem_copy(menu, menuitem):
    """Add a menuitem clone"""
    clone = wx.MenuItem(menu, menuitem.GetId(), menuitem.GetText(), menuitem.GetHelp(), menuitem.GetKind())
    if menuitem.GetBitmap().IsOk():
        clone.SetBitmap(menuitem.GetBitmap())
    menu.AppendItem(clone)
    if menuitem.GetKind() is wx.ITEM_CHECK:
        clone.Check(menuitem.IsChecked())


def set_menu_handlers(frame, imnu, command, update):
    """This method travels through the whole menu
    and submenu structure and set dispatch handlers.
    This method is used for dispatching events to focused
    windows that install the menus"""
    if type(imnu) is wx.Menu:
        for x in imnu.GetMenuItems():
            set_menu_handlers(frame, x, command, update)
    if type(imnu) is wx.MenuItem:
        frame.Bind(wx.EVT_MENU, command, id=imnu.GetId())
        frame.Bind(wx.EVT_UPDATE_UI, update, id=imnu.GetId())


def unset_menu_handlers(frame, imnu, command, update):
    """This method travels through the whole menu
    and submenu structure and unset dispatch handlers.
    This method is used for dispatching events to focused
    windows that install the menus"""
    if type(imnu) is wx.Menu:
        for x in imnu.GetMenuItems():
            set_menu_handlers(frame, x, command, update)
    if type(imnu) is wx.MenuItem:
        frame.Unbind(wx.EVT_MENU, command, id=imnu.GetId())
        frame.Unbind(wx.EVT_UPDATE_UI, update, id=imnu.GetId())


def clone_mnu(imnu, parent=None, enabled=False, notitle=False, separator=False):
    """
    Clone a menu or menuitem recursively. If
    enabled is specified, disabled items are filtered.
    separator makes sense only when parent is not None.
    This case, an separator will be added before the first
    menuitem to be copied.
    """
    if type(imnu) is wx.Menu:
        # if we filter disabled elements, we need to update menu first
        if enabled:
            imnu.UpdateUI()
        # if parent is specified the menu items are copied
        # if not, new menu is created
        if parent:
            clone = parent
            if separator:
                pos = clone.GetMenuItemCount()
        else:
            title = (not notitle and imnu.GetTitle()) or wx.EmptyString
            style = imnu.GetStyle()
            separator = False
            clone = wx.Menu(title=title, style=style)
        subs = [clone_mnu(x, clone, enabled) for x in
            imnu.GetMenuItems() if not enabled or x.IsEnabled()]
        subs = [x for x in subs if x is not None]
        if len(subs) > 0:
            # remove begin/adjoint/end separators
            while len(subs) and subs[0].GetKind() == wx.ITEM_SEPARATOR:
                clone.DestroyItem(subs[0])
                del subs[0]
            while len(subs) and subs[-1].GetKind() == wx.ITEM_SEPARATOR:
                clone.DestroyItem(subs[-1])
                del subs[-1]
            for i in range(len(subs) - 1, -1, -1):
                # remove duplicated separators
                if subs[i].GetKind() != wx.ITEM_SEPARATOR:
                    continue
                if subs[i - 1].GetKind() != wx.ITEM_SEPARATOR:
                    continue
                clone.DestroyItem(subs[i])
                del subs[i]
            if len(subs) > 0:
                if separator:
                    clone.InsertSeparator(pos)
                return clone
        if not parent:
            del clone
        return None
    if type(imnu) is wx.MenuItem:
        if enabled and not imnu.IsEnabled():
            return None
        simnu = imnu.GetSubMenu()
        if simnu is not None:
            simnu = clone_mnu(simnu, None, enabled)
        clone = wx.MenuItem(parent, imnu.GetId(), imnu.GetText(), imnu.GetHelp(), imnu.GetKind(), simnu)
        if imnu.GetBitmap().IsOk():
            clone.SetBitmap(imnu.GetBitmap())
        if parent is not None:
            parent.AppendItem(clone)
        if imnu.GetKind() is wx.ITEM_CHECK:
            clone.Check(imnu.IsChecked())
        return clone
